ip access-list test
  10 permit ip 192.0.2.1/32 any log
  20 permit ip 192.0.2.2/32 any log
  30 permit ip 192.0.2.3/32 any log
  40 permit ip 192.0.2.4/32 any log
  50 permit ip 192.0.2.5/32 any log
