
* {
    -webkit-box-sizing: border-box;
    -moz-box-sizing: border-box;
    box-sizing: border-box;
}

article, aside, details, figcaption, figure, footer, header, hgroup, nav, section {
    display: block;
}

audio, canvas, video {
    display: inline-block;
    *display: inline;
    *zoom: 1;
}

audio:not([controls]) {
    display: none;
}

[hidden] {
    display: none;
}

* {
    -webkit-box-sizing: border-box;
    -moz-box-sizing: border-box;
    box-sizing: border-box;
}

html {
    font-size: 100%;
    -webkit-text-size-adjust: 100%;
    -ms-text-size-adjust: 100%;
}

body {
    margin: 0;
}

a:hover, a:active {
    outline: 0;
}

abbr[title] {
    border-bottom: 1px dotted;
}

b, strong {
    font-weight: bold;
}

blockquote {
    margin: 0;
}

dfn {
    font-style: italic;
}

hr {
    display: block;
    height: 1px;
    border: 0;
    border-top: 1px solid #ccc;
    margin: 20px 0;
    padding: 0;
}

ins {
    background: #ff9;
    color: #000;
    text-decoration: none;
}

mark {
    background: #ff0;
    color: #000;
    font-style: italic;
    font-weight: bold;
}

pre, code, .rst-content tt, kbd, samp {
    font-family: monospace, serif;
    _font-family: "courier new", monospace;
    font-size: 1em;
}

pre {
    white-space: pre;
}

q {
    quotes: none;
}

q:before, q:after {
    content: "";
    content: none;
}

small {
    font-size: 85%;
}

sub, sup {
    font-size: 75%;
    line-height: 0;
    position: relative;
    vertical-align: baseline;
}

sup {
    top: -0.5em;
}

sub {
    bottom: -0.25em;
}

ul, ol, dl {
    margin: 0;
    padding: 0;
    list-style: none;
    list-style-image: none;
}

li {
    list-style: none;
}

dd {
    margin: 0;
}

img {
    border: 0;
    -ms-interpolation-mode: bicubic;
    vertical-align: middle;
    max-width: 100%;
}

svg:not(:root) {
    overflow: hidden;
}

figure {
    margin: 0;
}

form {
    margin: 0;
}

fieldset {
    border: 0;
    margin: 0;
    padding: 0;
}

label {
    cursor: pointer;
}

legend {
    border: 0;
    *margin-left: -7px;
    padding: 0;
    white-space: normal;
}

button, input, select, textarea {
    font-size: 100%;
    margin: 0;
    vertical-align: baseline;
    *vertical-align: middle;
}

button, input {
    line-height: normal;
}

button, input[type="button"], input[type="reset"], input[type="submit"] {
    cursor: pointer;
    -webkit-appearance: button;
    *overflow: visible;
}

button[disabled], input[disabled] {
    cursor: default;
}

input[type="checkbox"], input[type="radio"] {
    box-sizing: border-box;
    padding: 0;
    *width: 13px;
    *height: 13px;
}

input[type="search"] {
    -webkit-appearance: textfield;
    -moz-box-sizing: content-box;
    -webkit-box-sizing: content-box;
    box-sizing: content-box;
}

input[type="search"]::-webkit-search-decoration, input[type="search"]::-webkit-search-cancel-button {
    -webkit-appearance: none;
}

button::-moz-focus-inner, input::-moz-focus-inner {
    border: 0;
    padding: 0;
}

textarea {
    overflow: auto;
    vertical-align: top;
    resize: vertical;
}

table {
    border-collapse: collapse;
    border-spacing: 0;
}

td {
    vertical-align: top;
}

.chromeframe {
    margin: 0.2em 0;
    background: #ccc;
    color: #000;
    padding: 0.2em 0;
}

.ir {
    display: block;
    border: 0;
    text-indent: -999em;
    overflow: hidden;
    background-color: transparent;
    background-repeat: no-repeat;
    text-align: left;
    direction: ltr;
    *line-height: 0;
}

.ir br {
    display: none;
}

.hidden {
    display: none !important;
    visibility: hidden;
}

.visuallyhidden {
    border: 0;
    clip: rect(0 0 0 0);
    height: 1px;
    margin: -1px;
    overflow: hidden;
    padding: 0;
    position: absolute;
    width: 1px;
}

.visuallyhidden.focusable:active, .visuallyhidden.focusable:focus {
    clip: auto;
    height: auto;
    margin: 0;
    overflow: visible;
    position: static;
    width: auto;
}

.invisible {
    visibility: hidden;
}

.relative {
    position: relative;
}

big, small {
    font-size: 100%;
}

@media print {
    html, body, section {
        background: none !important;
    }

    * {
        box-shadow: none !important;
        text-shadow: none !important;
        filter: none !important;
        -ms-filter: none !important;
    }

    a, a:visited {
        text-decoration: underline;
    }

    .ir a:after, a[href^="javascript:"]:after, a[href^="#"]:after {
        content: "";
    }

    pre, blockquote {
        page-break-inside: avoid;
    }

    thead {
        display: table-header-group;
    }

    tr, img {
        page-break-inside: avoid;
    }

    img {
        max-width: 100% !important;
    }

    @page {
        margin: 0.5cm;
    }

    p, h2, h3 {
        orphans: 3;
        widows: 3;
    }

    h2, h3 {
        page-break-after: avoid;
    }
}

.font-smooth, .icon:before, .wy-inline-validate.wy-inline-validate-success .wy-input-context:before, .wy-inline-validate.wy-inline-validate-danger .wy-input-context:before, .wy-inline-validate.wy-inline-validate-warning .wy-input-context:before, .wy-inline-validate.wy-inline-validate-info .wy-input-context:before, .wy-tag-input-group .wy-tag .wy-tag-remove:before, .rst-content .admonition-title:before, .rst-content h1 .headerlink:before, .rst-content h2 .headerlink:before, .rst-content h3 .headerlink:before, .rst-content h4 .headerlink:before, .rst-content h5 .headerlink:before, .rst-content h6 .headerlink:before, .rst-content dl dt .headerlink:before, .wy-alert, .rst-content .note, .rst-content .attention, .rst-content .caution, .rst-content .danger, .rst-content .error, .rst-content .hint, .rst-content .important, .rst-content .tip, .rst-content .warning, .btn, input[type="text"], input[type="password"], input[type="email"], input[type="url"], input[type="date"], input[type="month"], input[type="time"], input[type="datetime"], input[type="datetime-local"], input[type="week"], input[type="number"], input[type="search"], input[type="tel"], input[type="color"], select, textarea, .wy-tag-input-group, .wy-menu-vertical li.on a, .wy-menu-vertical li.current>a, .wy-side-nav-search>a, .wy-side-nav-search .wy-dropdown>a, .wy-nav-top a {
    -webkit-font-smoothing: antialiased;
}

.clearfix {
    *zoom: 1;
}

.clearfix:before, .clearfix:after {
    display: table;
    content: "";
}

.clearfix:after {
    clear: both;
}

@font-face {
    font-family: fontawesome-webfont;
    font-weight: normal;
    font-style: normal;
    src: url("../font/fontawesome_webfont.eot");
    src: url("../font/fontawesome_webfont.eot?#iefix") format("embedded-opentype"), url("../font/fontawesome_webfont.woff") format("woff"), url("../font/fontawesome_webfont.ttf") format("truetype"), url("../font/fontawesome_webfont.svg#fontawesome-webfont") format("svg");
}

.icon:before, .wy-inline-validate.wy-inline-validate-success .wy-input-context:before, .wy-inline-validate.wy-inline-validate-danger .wy-input-context:before, .wy-inline-validate.wy-inline-validate-warning .wy-input-context:before, .wy-inline-validate.wy-inline-validate-info .wy-input-context:before, .wy-tag-input-group .wy-tag .wy-tag-remove:before, .rst-content .admonition-title:before, .rst-content h1 .headerlink:before, .rst-content h2 .headerlink:before, .rst-content h3 .headerlink:before, .rst-content h4 .headerlink:before, .rst-content h5 .headerlink:before, .rst-content h6 .headerlink:before, .rst-content dl dt .headerlink:before {
    display: inline-block;
    font-family: fontawesome-webfont;
    font-style: normal;
    font-weight: normal;
    line-height: 1;
    text-decoration: inherit;
}

a .icon, a .wy-inline-validate.wy-inline-validate-success .wy-input-context, .wy-inline-validate.wy-inline-validate-success a .wy-input-context, a .wy-inline-validate.wy-inline-validate-danger .wy-input-context, .wy-inline-validate.wy-inline-validate-danger a .wy-input-context, a .wy-inline-validate.wy-inline-validate-warning .wy-input-context, .wy-inline-validate.wy-inline-validate-warning a .wy-input-context, a .wy-inline-validate.wy-inline-validate-info .wy-input-context, .wy-inline-validate.wy-inline-validate-info a .wy-input-context, a .wy-tag-input-group .wy-tag .wy-tag-remove, .wy-tag-input-group .wy-tag a .wy-tag-remove, a .rst-content .admonition-title, .rst-content a .admonition-title, a .rst-content h1 .headerlink, .rst-content h1 a .headerlink, a .rst-content h2 .headerlink, .rst-content h2 a .headerlink, a .rst-content h3 .headerlink, .rst-content h3 a .headerlink, a .rst-content h4 .headerlink, .rst-content h4 a .headerlink, a .rst-content h5 .headerlink, .rst-content h5 a .headerlink, a .rst-content h6 .headerlink, .rst-content h6 a .headerlink, a .rst-content dl dt .headerlink, .rst-content dl dt a .headerlink {
    display: inline-block;
    text-decoration: inherit;
}

.icon-large:before {
    vertical-align: -10%;
    font-size: 1.33333em;
}

.btn .icon, .btn .wy-inline-validate.wy-inline-validate-success .wy-input-context, .wy-inline-validate.wy-inline-validate-success .btn .wy-input-context, .btn .wy-inline-validate.wy-inline-validate-danger .wy-input-context, .wy-inline-validate.wy-inline-validate-danger .btn .wy-input-context, .btn .wy-inline-validate.wy-inline-validate-warning .wy-input-context, .wy-inline-validate.wy-inline-validate-warning .btn .wy-input-context, .btn .wy-inline-validate.wy-inline-validate-info .wy-input-context, .wy-inline-validate.wy-inline-validate-info .btn .wy-input-context, .btn .wy-tag-input-group .wy-tag .wy-tag-remove, .wy-tag-input-group .wy-tag .btn .wy-tag-remove, .btn .rst-content .admonition-title, .rst-content .btn .admonition-title, .btn .rst-content h1 .headerlink, .rst-content h1 .btn .headerlink, .btn .rst-content h2 .headerlink, .rst-content h2 .btn .headerlink, .btn .rst-content h3 .headerlink, .rst-content h3 .btn .headerlink, .btn .rst-content h4 .headerlink, .rst-content h4 .btn .headerlink, .btn .rst-content h5 .headerlink, .rst-content h5 .btn .headerlink, .btn .rst-content h6 .headerlink, .rst-content h6 .btn .headerlink, .btn .rst-content dl dt .headerlink, .rst-content dl dt .btn .headerlink, .nav .icon, .nav .wy-inline-validate.wy-inline-validate-success .wy-input-context, .wy-inline-validate.wy-inline-validate-success .nav .wy-input-context, .nav .wy-inline-validate.wy-inline-validate-danger .wy-input-context, .wy-inline-validate.wy-inline-validate-danger .nav .wy-input-context, .nav .wy-inline-validate.wy-inline-validate-warning .wy-input-context, .wy-inline-validate.wy-inline-validate-warning .nav .wy-input-context, .nav .wy-inline-validate.wy-inline-validate-info .wy-input-context, .wy-inline-validate.wy-inline-validate-info .nav .wy-input-context, .nav .wy-tag-input-group .wy-tag .wy-tag-remove, .wy-tag-input-group .wy-tag .nav .wy-tag-remove, .nav .rst-content .admonition-title, .rst-content .nav .admonition-title, .nav .rst-content h1 .headerlink, .rst-content h1 .nav .headerlink, .nav .rst-content h2 .headerlink, .rst-content h2 .nav .headerlink, .nav .rst-content h3 .headerlink, .rst-content h3 .nav .headerlink, .nav .rst-content h4 .headerlink, .rst-content h4 .nav .headerlink, .nav .rst-content h5 .headerlink, .rst-content h5 .nav .headerlink, .nav .rst-content h6 .headerlink, .rst-content h6 .nav .headerlink, .nav .rst-content dl dt .headerlink, .rst-content dl dt .nav .headerlink {
    display: inline;
}

.btn .icon.icon-large, .btn .wy-inline-validate.wy-inline-validate-success .icon-large.wy-input-context, .wy-inline-validate.wy-inline-validate-success .btn .icon-large.wy-input-context, .btn .wy-inline-validate.wy-inline-validate-danger .icon-large.wy-input-context, .wy-inline-validate.wy-inline-validate-danger .btn .icon-large.wy-input-context, .btn .wy-inline-validate.wy-inline-validate-warning .icon-large.wy-input-context, .wy-inline-validate.wy-inline-validate-warning .btn .icon-large.wy-input-context, .btn .wy-inline-validate.wy-inline-validate-info .icon-large.wy-input-context, .wy-inline-validate.wy-inline-validate-info .btn .icon-large.wy-input-context, .btn .wy-tag-input-group .wy-tag .icon-large.wy-tag-remove, .wy-tag-input-group .wy-tag .btn .icon-large.wy-tag-remove, .btn .rst-content .icon-large.admonition-title, .rst-content .btn .icon-large.admonition-title, .btn .rst-content h1 .icon-large.headerlink, .rst-content h1 .btn .icon-large.headerlink, .btn .rst-content h2 .icon-large.headerlink, .rst-content h2 .btn .icon-large.headerlink, .btn .rst-content h3 .icon-large.headerlink, .rst-content h3 .btn .icon-large.headerlink, .btn .rst-content h4 .icon-large.headerlink, .rst-content h4 .btn .icon-large.headerlink, .btn .rst-content h5 .icon-large.headerlink, .rst-content h5 .btn .icon-large.headerlink, .btn .rst-content h6 .icon-large.headerlink, .rst-content h6 .btn .icon-large.headerlink, .btn .rst-content dl dt .icon-large.headerlink, .rst-content dl dt .btn .icon-large.headerlink, .nav .icon.icon-large, .nav .wy-inline-validate.wy-inline-validate-success .icon-large.wy-input-context, .wy-inline-validate.wy-inline-validate-success .nav .icon-large.wy-input-context, .nav .wy-inline-validate.wy-inline-validate-danger .icon-large.wy-input-context, .wy-inline-validate.wy-inline-validate-danger .nav .icon-large.wy-input-context, .nav .wy-inline-validate.wy-inline-validate-warning .icon-large.wy-input-context, .wy-inline-validate.wy-inline-validate-warning .nav .icon-large.wy-input-context, .nav .wy-inline-validate.wy-inline-validate-info .icon-large.wy-input-context, .wy-inline-validate.wy-inline-validate-info .nav .icon-large.wy-input-context, .nav .wy-tag-input-group .wy-tag .icon-large.wy-tag-remove, .wy-tag-input-group .wy-tag .nav .icon-large.wy-tag-remove, .nav .rst-content .icon-large.admonition-title, .rst-content .nav .icon-large.admonition-title, .nav .rst-content h1 .icon-large.headerlink, .rst-content h1 .nav .icon-large.headerlink, .nav .rst-content h2 .icon-large.headerlink, .rst-content h2 .nav .icon-large.headerlink, .nav .rst-content h3 .icon-large.headerlink, .rst-content h3 .nav .icon-large.headerlink, .nav .rst-content h4 .icon-large.headerlink, .rst-content h4 .nav .icon-large.headerlink, .nav .rst-content h5 .icon-large.headerlink, .rst-content h5 .nav .icon-large.headerlink, .nav .rst-content h6 .icon-large.headerlink, .rst-content h6 .nav .icon-large.headerlink, .nav .rst-content dl dt .icon-large.headerlink, .rst-content dl dt .nav .icon-large.headerlink {
    line-height: 0.9em;
}

.btn .icon.icon-spin, .btn .wy-inline-validate.wy-inline-validate-success .icon-spin.wy-input-context, .wy-inline-validate.wy-inline-validate-success .btn .icon-spin.wy-input-context, .btn .wy-inline-validate.wy-inline-validate-danger .icon-spin.wy-input-context, .wy-inline-validate.wy-inline-validate-danger .btn .icon-spin.wy-input-context, .btn .wy-inline-validate.wy-inline-validate-warning .icon-spin.wy-input-context, .wy-inline-validate.wy-inline-validate-warning .btn .icon-spin.wy-input-context, .btn .wy-inline-validate.wy-inline-validate-info .icon-spin.wy-input-context, .wy-inline-validate.wy-inline-validate-info .btn .icon-spin.wy-input-context, .btn .wy-tag-input-group .wy-tag .icon-spin.wy-tag-remove, .wy-tag-input-group .wy-tag .btn .icon-spin.wy-tag-remove, .btn .rst-content .icon-spin.admonition-title, .rst-content .btn .icon-spin.admonition-title, .btn .rst-content h1 .icon-spin.headerlink, .rst-content h1 .btn .icon-spin.headerlink, .btn .rst-content h2 .icon-spin.headerlink, .rst-content h2 .btn .icon-spin.headerlink, .btn .rst-content h3 .icon-spin.headerlink, .rst-content h3 .btn .icon-spin.headerlink, .btn .rst-content h4 .icon-spin.headerlink, .rst-content h4 .btn .icon-spin.headerlink, .btn .rst-content h5 .icon-spin.headerlink, .rst-content h5 .btn .icon-spin.headerlink, .btn .rst-content h6 .icon-spin.headerlink, .rst-content h6 .btn .icon-spin.headerlink, .btn .rst-content dl dt .icon-spin.headerlink, .rst-content dl dt .btn .icon-spin.headerlink, .nav .icon.icon-spin, .nav .wy-inline-validate.wy-inline-validate-success .icon-spin.wy-input-context, .wy-inline-validate.wy-inline-validate-success .nav .icon-spin.wy-input-context, .nav .wy-inline-validate.wy-inline-validate-danger .icon-spin.wy-input-context, .wy-inline-validate.wy-inline-validate-danger .nav .icon-spin.wy-input-context, .nav .wy-inline-validate.wy-inline-validate-warning .icon-spin.wy-input-context, .wy-inline-validate.wy-inline-validate-warning .nav .icon-spin.wy-input-context, .nav .wy-inline-validate.wy-inline-validate-info .icon-spin.wy-input-context, .wy-inline-validate.wy-inline-validate-info .nav .icon-spin.wy-input-context, .nav .wy-tag-input-group .wy-tag .icon-spin.wy-tag-remove, .wy-tag-input-group .wy-tag .nav .icon-spin.wy-tag-remove, .nav .rst-content .icon-spin.admonition-title, .rst-content .nav .icon-spin.admonition-title, .nav .rst-content h1 .icon-spin.headerlink, .rst-content h1 .nav .icon-spin.headerlink, .nav .rst-content h2 .icon-spin.headerlink, .rst-content h2 .nav .icon-spin.headerlink, .nav .rst-content h3 .icon-spin.headerlink, .rst-content h3 .nav .icon-spin.headerlink, .nav .rst-content h4 .icon-spin.headerlink, .rst-content h4 .nav .icon-spin.headerlink, .nav .rst-content h5 .icon-spin.headerlink, .rst-content h5 .nav .icon-spin.headerlink, .nav .rst-content h6 .icon-spin.headerlink, .rst-content h6 .nav .icon-spin.headerlink, .nav .rst-content dl dt .icon-spin.headerlink, .rst-content dl dt .nav .icon-spin.headerlink {
    display: inline-block;
}

.btn.icon:before, .wy-inline-validate.wy-inline-validate-success .btn.wy-input-context:before, .wy-inline-validate.wy-inline-validate-danger .btn.wy-input-context:before, .wy-inline-validate.wy-inline-validate-warning .btn.wy-input-context:before, .wy-inline-validate.wy-inline-validate-info .btn.wy-input-context:before, .wy-tag-input-group .wy-tag .btn.wy-tag-remove:before, .rst-content .btn.admonition-title:before, .rst-content h1 .btn.headerlink:before, .rst-content h2 .btn.headerlink:before, .rst-content h3 .btn.headerlink:before, .rst-content h4 .btn.headerlink:before, .rst-content h5 .btn.headerlink:before, .rst-content h6 .btn.headerlink:before, .rst-content dl dt .btn.headerlink:before {
    opacity: 0.5;
    -webkit-transition: opacity 0.05s ease-in;
    -moz-transition: opacity 0.05s ease-in;
    transition: opacity 0.05s ease-in;
}

.btn.icon:hover:before, .wy-inline-validate.wy-inline-validate-success .btn.wy-input-context:hover:before, .wy-inline-validate.wy-inline-validate-danger .btn.wy-input-context:hover:before, .wy-inline-validate.wy-inline-validate-warning .btn.wy-input-context:hover:before, .wy-inline-validate.wy-inline-validate-info .btn.wy-input-context:hover:before, .wy-tag-input-group .wy-tag .btn.wy-tag-remove:hover:before, .rst-content .btn.admonition-title:hover:before, .rst-content h1 .btn.headerlink:hover:before, .rst-content h2 .btn.headerlink:hover:before, .rst-content h3 .btn.headerlink:hover:before, .rst-content h4 .btn.headerlink:hover:before, .rst-content h5 .btn.headerlink:hover:before, .rst-content h6 .btn.headerlink:hover:before, .rst-content dl dt .btn.headerlink:hover:before {
    opacity: 1;
}

.btn-mini .icon:before, .btn-mini .wy-inline-validate.wy-inline-validate-success .wy-input-context:before, .wy-inline-validate.wy-inline-validate-success .btn-mini .wy-input-context:before, .btn-mini .wy-inline-validate.wy-inline-validate-danger .wy-input-context:before, .wy-inline-validate.wy-inline-validate-danger .btn-mini .wy-input-context:before, .btn-mini .wy-inline-validate.wy-inline-validate-warning .wy-input-context:before, .wy-inline-validate.wy-inline-validate-warning .btn-mini .wy-input-context:before, .btn-mini .wy-inline-validate.wy-inline-validate-info .wy-input-context:before, .wy-inline-validate.wy-inline-validate-info .btn-mini .wy-input-context:before, .btn-mini .wy-tag-input-group .wy-tag .wy-tag-remove:before, .wy-tag-input-group .wy-tag .btn-mini .wy-tag-remove:before, .btn-mini .rst-content .admonition-title:before, .rst-content .btn-mini .admonition-title:before, .btn-mini .rst-content h1 .headerlink:before, .rst-content h1 .btn-mini .headerlink:before, .btn-mini .rst-content h2 .headerlink:before, .rst-content h2 .btn-mini .headerlink:before, .btn-mini .rst-content h3 .headerlink:before, .rst-content h3 .btn-mini .headerlink:before, .btn-mini .rst-content h4 .headerlink:before, .rst-content h4 .btn-mini .headerlink:before, .btn-mini .rst-content h5 .headerlink:before, .rst-content h5 .btn-mini .headerlink:before, .btn-mini .rst-content h6 .headerlink:before, .rst-content h6 .btn-mini .headerlink:before, .btn-mini .rst-content dl dt .headerlink:before, .rst-content dl dt .btn-mini .headerlink:before {
    font-size: 14px;
    vertical-align: -15%;
}

li .icon, li .wy-inline-validate.wy-inline-validate-success .wy-input-context, .wy-inline-validate.wy-inline-validate-success li .wy-input-context, li .wy-inline-validate.wy-inline-validate-danger .wy-input-context, .wy-inline-validate.wy-inline-validate-danger li .wy-input-context, li .wy-inline-validate.wy-inline-validate-warning .wy-input-context, .wy-inline-validate.wy-inline-validate-warning li .wy-input-context, li .wy-inline-validate.wy-inline-validate-info .wy-input-context, .wy-inline-validate.wy-inline-validate-info li .wy-input-context, li .wy-tag-input-group .wy-tag .wy-tag-remove, .wy-tag-input-group .wy-tag li .wy-tag-remove, li .rst-content .admonition-title, .rst-content li .admonition-title, li .rst-content h1 .headerlink, .rst-content h1 li .headerlink, li .rst-content h2 .headerlink, .rst-content h2 li .headerlink, li .rst-content h3 .headerlink, .rst-content h3 li .headerlink, li .rst-content h4 .headerlink, .rst-content h4 li .headerlink, li .rst-content h5 .headerlink, .rst-content h5 li .headerlink, li .rst-content h6 .headerlink, .rst-content h6 li .headerlink, li .rst-content dl dt .headerlink, .rst-content dl dt li .headerlink {
    display: inline-block;
}

li .icon-large:before, li .icon-large:before {
    width: 1.875em;
}

ul.icons {
    list-style-type: none;
    margin-left: 2em;
    text-indent: -0.8em;
}

ul.icons li .icon, ul.icons li .wy-inline-validate.wy-inline-validate-success .wy-input-context, .wy-inline-validate.wy-inline-validate-success ul.icons li .wy-input-context, ul.icons li .wy-inline-validate.wy-inline-validate-danger .wy-input-context, .wy-inline-validate.wy-inline-validate-danger ul.icons li .wy-input-context, ul.icons li .wy-inline-validate.wy-inline-validate-warning .wy-input-context, .wy-inline-validate.wy-inline-validate-warning ul.icons li .wy-input-context, ul.icons li .wy-inline-validate.wy-inline-validate-info .wy-input-context, .wy-inline-validate.wy-inline-validate-info ul.icons li .wy-input-context, ul.icons li .wy-tag-input-group .wy-tag .wy-tag-remove, .wy-tag-input-group .wy-tag ul.icons li .wy-tag-remove, ul.icons li .rst-content .admonition-title, .rst-content ul.icons li .admonition-title, ul.icons li .rst-content h1 .headerlink, .rst-content h1 ul.icons li .headerlink, ul.icons li .rst-content h2 .headerlink, .rst-content h2 ul.icons li .headerlink, ul.icons li .rst-content h3 .headerlink, .rst-content h3 ul.icons li .headerlink, ul.icons li .rst-content h4 .headerlink, .rst-content h4 ul.icons li .headerlink, ul.icons li .rst-content h5 .headerlink, .rst-content h5 ul.icons li .headerlink, ul.icons li .rst-content h6 .headerlink, .rst-content h6 ul.icons li .headerlink, ul.icons li .rst-content dl dt .headerlink, .rst-content dl dt ul.icons li .headerlink {
    width: 0.8em;
}

ul.icons li .icon-large:before, ul.icons li .icon-large:before {
    vertical-align: baseline;
}

.icon-glass:before {
    content: "\f000";
}

.icon-music:before {
    content: "\f001";
}

.icon-search:before {
    content: "\f002";
}

.icon-envelope-alt:before {
    content: "\f003";
}

.icon-heart:before {
    content: "\f004";
}

.icon-star:before {
    content: "\f005";
}

.icon-star-empty:before {
    content: "\f006";
}

.icon-user:before {
    content: "\f007";
}

.icon-film:before {
    content: "\f008";
}

.icon-th-large:before {
    content: "\f009";
}

.icon-th:before {
    content: "\f00a";
}

.icon-th-list:before {
    content: "\f00b";
}

.icon-ok:before {
    content: "\f00c";
}

.icon-remove:before, .wy-tag-input-group .wy-tag .wy-tag-remove:before {
    content: "\f00d";
}

.icon-zoom-in:before {
    content: "\f00e";
}

.icon-zoom-out:before {
    content: "\f010";
}

.icon-power-off:before, .icon-off:before {
    content: "\f011";
}

.icon-signal:before {
    content: "\f012";
}

.icon-gear:before, .icon-cog:before {
    content: "\f013";
}

.icon-trash:before {
    content: "\f014";
}

.icon-home:before {
    content: "\f015";
}

.icon-file-alt:before {
    content: "\f016";
}

.icon-time:before {
    content: "\f017";
}

.icon-road:before {
    content: "\f018";
}

.icon-download-alt:before {
    content: "\f019";
}

.icon-download:before {
    content: "\f01a";
}

.icon-upload:before {
    content: "\f01b";
}

.icon-inbox:before {
    content: "\f01c";
}

.icon-play-circle:before {
    content: "\f01d";
}

.icon-rotate-right:before, .icon-repeat:before {
    content: "\f01e";
}

.icon-refresh:before {
    content: "\f021";
}

.icon-list-alt:before {
    content: "\f022";
}

.icon-lock:before {
    content: "\f023";
}

.icon-flag:before {
    content: "\f024";
}

.icon-headphones:before {
    content: "\f025";
}

.icon-volume-off:before {
    content: "\f026";
}

.icon-volume-down:before {
    content: "\f027";
}

.icon-volume-up:before {
    content: "\f028";
}

.icon-qrcode:before {
    content: "\f029";
}

.icon-barcode:before {
    content: "\f02a";
}

.icon-tag:before {
    content: "\f02b";
}

.icon-tags:before {
    content: "\f02c";
}

.icon-book:before {
    content: "\f02d";
}

.icon-bookmark:before {
    content: "\f02e";
}

.icon-print:before {
    content: "\f02f";
}

.icon-camera:before {
    content: "\f030";
}

.icon-font:before {
    content: "\f031";
}

.icon-bold:before {
    content: "\f032";
}

.icon-italic:before {
    content: "\f033";
}

.icon-text-height:before {
    content: "\f034";
}

.icon-text-width:before {
    content: "\f035";
}

.icon-align-left:before {
    content: "\f036";
}

.icon-align-center:before {
    content: "\f037";
}

.icon-align-right:before {
    content: "\f038";
}

.icon-align-justify:before {
    content: "\f039";
}

.icon-list:before {
    content: "\f03a";
}

.icon-indent-left:before {
    content: "\f03b";
}

.icon-indent-right:before {
    content: "\f03c";
}

.icon-facetime-video:before {
    content: "\f03d";
}

.icon-picture:before {
    content: "\f03e";
}

.icon-pencil:before {
    content: "\f040";
}

.icon-map-marker:before {
    content: "\f041";
}

.icon-adjust:before {
    content: "\f042";
}

.icon-tint:before {
    content: "\f043";
}

.icon-edit:before {
    content: "\f044";
}

.icon-share:before {
    content: "\f045";
}

.icon-check:before {
    content: "\f046";
}

.icon-move:before {
    content: "\f047";
}

.icon-step-backward:before {
    content: "\f048";
}

.icon-fast-backward:before {
    content: "\f049";
}

.icon-backward:before {
    content: "\f04a";
}

.icon-play:before {
    content: "\f04b";
}

.icon-pause:before {
    content: "\f04c";
}

.icon-stop:before {
    content: "\f04d";
}

.icon-forward:before {
    content: "\f04e";
}

.icon-fast-forward:before {
    content: "\f050";
}

.icon-step-forward:before {
    content: "\f051";
}

.icon-eject:before {
    content: "\f052";
}

.icon-chevron-left:before {
    content: "\f053";
}

.icon-chevron-right:before {
    content: "\f054";
}

.icon-plus-sign:before {
    content: "\f055";
}

.icon-minus-sign:before {
    content: "\f056";
}

.icon-remove-sign:before, .wy-inline-validate.wy-inline-validate-danger .wy-input-context:before {
    content: "\f057";
}

.icon-ok-sign:before {
    content: "\f058";
}

.icon-question-sign:before {
    content: "\f059";
}

.icon-info-sign:before {
    content: "\f05a";
}

.icon-screenshot:before {
    content: "\f05b";
}

.icon-remove-circle:before {
    content: "\f05c";
}

.icon-ok-circle:before {
    content: "\f05d";
}

.icon-ban-circle:before {
    content: "\f05e";
}

.icon-arrow-left:before {
    content: "\f060";
}

.icon-arrow-right:before {
    content: "\f061";
}

.icon-arrow-up:before {
    content: "\f062";
}

.icon-arrow-down:before {
    content: "\f063";
}

.icon-mail-forward:before, .icon-share-alt:before {
    content: "\f064";
}

.icon-resize-full:before {
    content: "\f065";
}

.icon-resize-small:before {
    content: "\f066";
}

.icon-plus:before {
    content: "\f067";
}

.icon-minus:before {
    content: "\f068";
}

.icon-asterisk:before {
    content: "\f069";
}

.icon-exclamation-sign:before, .wy-inline-validate.wy-inline-validate-warning .wy-input-context:before, .wy-inline-validate.wy-inline-validate-info .wy-input-context:before, .rst-content .admonition-title:before {
    content: "\f06a";
}

.icon-gift:before {
    content: "\f06b";
}

.icon-leaf:before {
    content: "\f06c";
}

.icon-fire:before {
    content: "\f06d";
}

.icon-eye-open:before {
    content: "\f06e";
}

.icon-eye-close:before {
    content: "\f070";
}

.icon-warning-sign:before {
    content: "\f071";
}

.icon-plane:before {
    content: "\f072";
}

.icon-calendar:before {
    content: "\f073";
}

.icon-random:before {
    content: "\f074";
}

.icon-comment:before {
    content: "\f075";
}

.icon-magnet:before {
    content: "\f076";
}

.icon-chevron-up:before {
    content: "\f077";
}

.icon-chevron-down:before {
    content: "\f078";
}

.icon-retweet:before {
    content: "\f079";
}

.icon-shopping-cart:before {
    content: "\f07a";
}

.icon-folder-close:before {
    content: "\f07b";
}

.icon-folder-open:before {
    content: "\f07c";
}

.icon-resize-vertical:before {
    content: "\f07d";
}

.icon-resize-horizontal:before {
    content: "\f07e";
}

.icon-bar-chart:before {
    content: "\f080";
}

.icon-twitter-sign:before {
    content: "\f081";
}

.icon-facebook-sign:before {
    content: "\f082";
}

.icon-camera-retro:before {
    content: "\f083";
}

.icon-key:before {
    content: "\f084";
}

.icon-gears:before, .icon-cogs:before {
    content: "\f085";
}

.icon-comments:before {
    content: "\f086";
}

.icon-thumbs-up-alt:before {
    content: "\f087";
}

.icon-thumbs-down-alt:before {
    content: "\f088";
}

.icon-star-half:before {
    content: "\f089";
}

.icon-heart-empty:before {
    content: "\f08a";
}

.icon-signout:before {
    content: "\f08b";
}

.icon-linkedin-sign:before {
    content: "\f08c";
}

.icon-pushpin:before {
    content: "\f08d";
}

.icon-external-link:before {
    content: "\f08e";
}

.icon-signin:before {
    content: "\f090";
}

.icon-trophy:before {
    content: "\f091";
}

.icon-github-sign:before {
    content: "\f092";
}

.icon-upload-alt:before {
    content: "\f093";
}

.icon-lemon:before {
    content: "\f094";
}

.icon-phone:before {
    content: "\f095";
}

.icon-unchecked:before, .icon-check-empty:before {
    content: "\f096";
}

.icon-bookmark-empty:before {
    content: "\f097";
}

.icon-phone-sign:before {
    content: "\f098";
}

.icon-twitter:before {
    content: "\f099";
}

.icon-facebook:before {
    content: "\f09a";
}

.icon-github:before {
    content: "\f09b";
}

.icon-unlock:before {
    content: "\f09c";
}

.icon-credit-card:before {
    content: "\f09d";
}

.icon-rss:before {
    content: "\f09e";
}

.icon-hdd:before {
    content: "\f0a0";
}

.icon-bullhorn:before {
    content: "\f0a1";
}

.icon-bell:before {
    content: "\f0a2";
}

.icon-certificate:before {
    content: "\f0a3";
}

.icon-hand-right:before {
    content: "\f0a4";
}

.icon-hand-left:before {
    content: "\f0a5";
}

.icon-hand-up:before {
    content: "\f0a6";
}

.icon-hand-down:before {
    content: "\f0a7";
}

.icon-circle-arrow-left:before {
    content: "\f0a8";
}

.icon-circle-arrow-right:before {
    content: "\f0a9";
}

.icon-circle-arrow-up:before {
    content: "\f0aa";
}

.icon-circle-arrow-down:before {
    content: "\f0ab";
}

.icon-globe:before {
    content: "\f0ac";
}

.icon-wrench:before {
    content: "\f0ad";
}

.icon-tasks:before {
    content: "\f0ae";
}

.icon-filter:before {
    content: "\f0b0";
}

.icon-briefcase:before {
    content: "\f0b1";
}

.icon-fullscreen:before {
    content: "\f0b2";
}

.icon-group:before {
    content: "\f0c0";
}

.icon-link:before {
    content: "\f0c1";
}

.icon-cloud:before {
    content: "\f0c2";
}

.icon-beaker:before {
    content: "\f0c3";
}

.icon-cut:before {
    content: "\f0c4";
}

.icon-copy:before {
    content: "\f0c5";
}

.icon-paperclip:before, .icon-paper-clip:before {
    content: "\f0c6";
}

.icon-save:before {
    content: "\f0c7";
}

.icon-sign-blank:before {
    content: "\f0c8";
}

.icon-reorder:before {
    content: "\f0c9";
}

.icon-list-ul:before {
    content: "\f0ca";
}

.icon-list-ol:before {
    content: "\f0cb";
}

.icon-strikethrough:before {
    content: "\f0cc";
}

.icon-underline:before {
    content: "\f0cd";
}

.icon-table:before {
    content: "\f0ce";
}

.icon-magic:before {
    content: "\f0d0";
}

.icon-truck:before {
    content: "\f0d1";
}

.icon-pinterest:before {
    content: "\f0d2";
}

.icon-pinterest-sign:before {
    content: "\f0d3";
}

.icon-google-plus-sign:before {
    content: "\f0d4";
}

.icon-google-plus:before {
    content: "\f0d5";
}

.icon-money:before {
    content: "\f0d6";
}

.icon-caret-down:before {
    content: "\f0d7";
}

.icon-caret-up:before {
    content: "\f0d8";
}

.icon-caret-left:before {
    content: "\f0d9";
}

.icon-caret-right:before {
    content: "\f0da";
}

.icon-columns:before {
    content: "\f0db";
}

.icon-sort:before {
    content: "\f0dc";
}

.icon-sort-down:before {
    content: "\f0dd";
}

.icon-sort-up:before {
    content: "\f0de";
}

.icon-envelope:before {
    content: "\f0e0";
}

.icon-linkedin:before {
    content: "\f0e1";
}

.icon-rotate-left:before, .icon-undo:before {
    content: "\f0e2";
}

.icon-legal:before {
    content: "\f0e3";
}

.icon-dashboard:before {
    content: "\f0e4";
}

.icon-comment-alt:before {
    content: "\f0e5";
}

.icon-comments-alt:before {
    content: "\f0e6";
}

.icon-bolt:before {
    content: "\f0e7";
}

.icon-sitemap:before {
    content: "\f0e8";
}

.icon-umbrella:before {
    content: "\f0e9";
}

.icon-paste:before {
    content: "\f0ea";
}

.icon-lightbulb:before {
    content: "\f0eb";
}

.icon-exchange:before {
    content: "\f0ec";
}

.icon-cloud-download:before {
    content: "\f0ed";
}

.icon-cloud-upload:before {
    content: "\f0ee";
}

.icon-user-md:before {
    content: "\f0f0";
}

.icon-stethoscope:before {
    content: "\f0f1";
}

.icon-suitcase:before {
    content: "\f0f2";
}

.icon-bell-alt:before {
    content: "\f0f3";
}

.icon-coffee:before {
    content: "\f0f4";
}

.icon-food:before {
    content: "\f0f5";
}

.icon-file-text-alt:before {
    content: "\f0f6";
}

.icon-building:before {
    content: "\f0f7";
}

.icon-hospital:before {
    content: "\f0f8";
}

.icon-ambulance:before {
    content: "\f0f9";
}

.icon-medkit:before {
    content: "\f0fa";
}

.icon-fighter-jet:before {
    content: "\f0fb";
}

.icon-beer:before {
    content: "\f0fc";
}

.icon-h-sign:before {
    content: "\f0fd";
}

.icon-plus-sign-alt:before {
    content: "\f0fe";
}

.icon-double-angle-left:before {
    content: "\f100";
}

.icon-double-angle-right:before {
    content: "\f101";
}

.icon-double-angle-up:before {
    content: "\f102";
}

.icon-double-angle-down:before {
    content: "\f103";
}

.icon-angle-left:before {
    content: "\f104";
}

.icon-angle-right:before {
    content: "\f105";
}

.icon-angle-up:before {
    content: "\f106";
}

.icon-angle-down:before {
    content: "\f107";
}

.icon-desktop:before {
    content: "\f108";
}

.icon-laptop:before {
    content: "\f109";
}

.icon-tablet:before {
    content: "\f10a";
}

.icon-mobile-phone:before {
    content: "\f10b";
}

.icon-circle-blank:before {
    content: "\f10c";
}

.icon-quote-left:before {
    content: "\f10d";
}

.icon-quote-right:before {
    content: "\f10e";
}

.icon-spinner:before {
    content: "\f110";
}

.icon-circle:before {
    content: "\f111";
}

.icon-mail-reply:before, .icon-reply:before {
    content: "\f112";
}

.icon-github-alt:before {
    content: "\f113";
}

.icon-folder-close-alt:before {
    content: "\f114";
}

.icon-folder-open-alt:before {
    content: "\f115";
}

.icon-expand-alt:before {
    content: "\f116";
}

.icon-collapse-alt:before {
    content: "\f117";
}

.icon-smile:before {
    content: "\f118";
}

.icon-frown:before {
    content: "\f119";
}

.icon-meh:before {
    content: "\f11a";
}

.icon-gamepad:before {
    content: "\f11b";
}

.icon-keyboard:before {
    content: "\f11c";
}

.icon-flag-alt:before {
    content: "\f11d";
}

.icon-flag-checkered:before {
    content: "\f11e";
}

.icon-terminal:before {
    content: "\f120";
}

.icon-code:before {
    content: "\f121";
}

.icon-reply-all:before {
    content: "\f122";
}

.icon-mail-reply-all:before {
    content: "\f122";
}

.icon-star-half-full:before, .icon-star-half-empty:before {
    content: "\f123";
}

.icon-location-arrow:before {
    content: "\f124";
}

.icon-crop:before {
    content: "\f125";
}

.icon-code-fork:before {
    content: "\f126";
}

.icon-unlink:before {
    content: "\f127";
}

.icon-question:before {
    content: "\f128";
}

.icon-info:before {
    content: "\f129";
}

.icon-exclamation:before {
    content: "\f12a";
}

.icon-superscript:before {
    content: "\f12b";
}

.icon-subscript:before {
    content: "\f12c";
}

.icon-eraser:before {
    content: "\f12d";
}

.icon-puzzle-piece:before {
    content: "\f12e";
}

.icon-microphone:before {
    content: "\f130";
}

.icon-microphone-off:before {
    content: "\f131";
}

.icon-shield:before {
    content: "\f132";
}

.icon-calendar-empty:before {
    content: "\f133";
}

.icon-fire-extinguisher:before {
    content: "\f134";
}

.icon-rocket:before {
    content: "\f135";
}

.icon-maxcdn:before {
    content: "\f136";
}

.icon-chevron-sign-left:before {
    content: "\f137";
}

.icon-chevron-sign-right:before {
    content: "\f138";
}

.icon-chevron-sign-up:before {
    content: "\f139";
}

.icon-chevron-sign-down:before {
    content: "\f13a";
}

.icon-html5:before {
    content: "\f13b";
}

.icon-css3:before {
    content: "\f13c";
}

.icon-anchor:before {
    content: "\f13d";
}

.icon-unlock-alt:before {
    content: "\f13e";
}

.icon-bullseye:before {
    content: "\f140";
}

.icon-ellipsis-horizontal:before {
    content: "\f141";
}

.icon-ellipsis-vertical:before {
    content: "\f142";
}

.icon-rss-sign:before {
    content: "\f143";
}

.icon-play-sign:before {
    content: "\f144";
}

.icon-ticket:before {
    content: "\f145";
}

.icon-minus-sign-alt:before {
    content: "\f146";
}

.icon-check-minus:before {
    content: "\f147";
}

.icon-level-up:before {
    content: "\f148";
}

.icon-level-down:before {
    content: "\f149";
}

.icon-check-sign:before, .wy-inline-validate.wy-inline-validate-success .wy-input-context:before {
    content: "\f14a";
}

.icon-edit-sign:before {
    content: "\f14b";
}

.icon-external-link-sign:before {
    content: "\f14c";
}

.icon-share-sign:before {
    content: "\f14d";
}

.icon-compass:before {
    content: "\f14e";
}

.icon-collapse:before {
    content: "\f150";
}

.icon-collapse-top:before {
    content: "\f151";
}

.icon-expand:before {
    content: "\f152";
}

.icon-euro:before, .icon-eur:before {
    content: "\f153";
}

.icon-gbp:before {
    content: "\f154";
}

.icon-dollar:before, .icon-usd:before {
    content: "\f155";
}

.icon-rupee:before, .icon-inr:before {
    content: "\f156";
}

.icon-yen:before, .icon-jpy:before {
    content: "\f157";
}

.icon-renminbi:before, .icon-cny:before {
    content: "\f158";
}

.icon-won:before, .icon-krw:before {
    content: "\f159";
}

.icon-bitcoin:before, .icon-btc:before {
    content: "\f15a";
}

.icon-file:before {
    content: "\f15b";
}

.icon-file-text:before {
    content: "\f15c";
}

.icon-sort-by-alphabet:before {
    content: "\f15d";
}

.icon-sort-by-alphabet-alt:before {
    content: "\f15e";
}

.icon-sort-by-attributes:before {
    content: "\f160";
}

.icon-sort-by-attributes-alt:before {
    content: "\f161";
}

.icon-sort-by-order:before {
    content: "\f162";
}

.icon-sort-by-order-alt:before {
    content: "\f163";
}

.icon-thumbs-up:before {
    content: "\f164";
}

.icon-thumbs-down:before {
    content: "\f165";
}

.icon-youtube-sign:before {
    content: "\f166";
}

.icon-youtube:before {
    content: "\f167";
}

.icon-xing:before {
    content: "\f168";
}

.icon-xing-sign:before {
    content: "\f169";
}

.icon-youtube-play:before {
    content: "\f16a";
}

.icon-dropbox:before {
    content: "\f16b";
}

.icon-stackexchange:before {
    content: "\f16c";
}

.icon-instagram:before {
    content: "\f16d";
}

.icon-flickr:before {
    content: "\f16e";
}

.icon-adn:before {
    content: "\f170";
}

.icon-bitbucket:before {
    content: "\f171";
}

.icon-bitbucket-sign:before {
    content: "\f172";
}

.icon-tumblr:before {
    content: "\f173";
}

.icon-tumblr-sign:before {
    content: "\f174";
}

.icon-long-arrow-down:before {
    content: "\f175";
}

.icon-long-arrow-up:before {
    content: "\f176";
}

.icon-long-arrow-left:before {
    content: "\f177";
}

.icon-long-arrow-right:before {
    content: "\f178";
}

.icon-apple:before {
    content: "\f179";
}

.icon-windows:before {
    content: "\f17a";
}

.icon-android:before {
    content: "\f17b";
}

.icon-linux:before {
    content: "\f17c";
}

.icon-dribbble:before {
    content: "\f17d";
}

.icon-skype:before {
    content: "\f17e";
}

.icon-foursquare:before {
    content: "\f180";
}

.icon-trello:before {
    content: "\f181";
}

.icon-female:before {
    content: "\f182";
}

.icon-male:before {
    content: "\f183";
}

.icon-gittip:before {
    content: "\f184";
}

.icon-sun:before {
    content: "\f185";
}

.icon-moon:before {
    content: "\f186";
}

.icon-archive:before {
    content: "\f187";
}

.icon-bug:before {
    content: "\f188";
}

.icon-vk:before {
    content: "\f189";
}

.icon-weibo:before {
    content: "\f18a";
}

.icon-renren:before {
    content: "\f18b";
}

.wy-alert, .rst-content .note, .rst-content .attention, .rst-content .caution, .rst-content .danger, .rst-content .error, .rst-content .hint, .rst-content .important, .rst-content .tip, .rst-content .warning {
    padding: 24px;
    line-height: 24px;
    margin-bottom: 24px;
    border-left: solid 3px transparent;
}

.wy-alert strong, .rst-content .note strong, .rst-content .attention strong, .rst-content .caution strong, .rst-content .danger strong, .rst-content .error strong, .rst-content .hint strong, .rst-content .important strong, .rst-content .tip strong, .rst-content .warning strong, .wy-alert a, .rst-content .note a, .rst-content .attention a, .rst-content .caution a, .rst-content .danger a, .rst-content .error a, .rst-content .hint a, .rst-content .important a, .rst-content .tip a, .rst-content .warning a {
    color: #fff;
}

.wy-alert.wy-alert-danger, .rst-content .wy-alert-danger.note, .rst-content .wy-alert-danger.attention, .rst-content .wy-alert-danger.caution, .rst-content .danger, .rst-content .error, .rst-content .wy-alert-danger.hint, .rst-content .wy-alert-danger.important, .rst-content .wy-alert-danger.tip, .rst-content .wy-alert-danger.warning {
    background: #e74c3c;
    color: #fff;
    border-color: #d62c1a;
}

.wy-alert.wy-alert-warning, .rst-content .wy-alert-warning.note, .rst-content .attention, .rst-content .caution, .rst-content .wy-alert-warning.danger, .rst-content .wy-alert-warning.error, .rst-content .wy-alert-warning.hint, .rst-content .wy-alert-warning.important, .rst-content .wy-alert-warning.tip, .rst-content .warning {
    background: #e67e22;
    color: #fff;
    border-color: #bf6516;
}

.wy-alert.wy-alert-info, .rst-content .note, .rst-content .wy-alert-info.attention, .rst-content .wy-alert-info.caution, .rst-content .wy-alert-info.danger, .rst-content .wy-alert-info.error, .rst-content .hint, .rst-content .important, .rst-content .tip, .rst-content .wy-alert-info.warning {
    background: #2980b9;
    color: #fff;
    border-color: #20638f;
}

.wy-alert.wy-alert-success, .rst-content .wy-alert-success.note, .rst-content .wy-alert-success.attention, .rst-content .wy-alert-success.caution, .rst-content .wy-alert-success.danger, .rst-content .wy-alert-success.error, .rst-content .wy-alert-success.hint, .rst-content .wy-alert-success.important, .rst-content .wy-alert-success.tip, .rst-content .wy-alert-success.warning {
    background: #27ae60;
    color: #fff;
    border-color: #1e8449;
}

.wy-alert.wy-alert-neutral, .rst-content .wy-alert-neutral.note, .rst-content .wy-alert-neutral.attention, .rst-content .wy-alert-neutral.caution, .rst-content .wy-alert-neutral.danger, .rst-content .wy-alert-neutral.error, .rst-content .wy-alert-neutral.hint, .rst-content .wy-alert-neutral.important, .rst-content .wy-alert-neutral.tip, .rst-content .wy-alert-neutral.warning {
    background: #f3f6f6;
    border-color: #e1e4e5;
}

.wy-alert.wy-alert-neutral strong, .rst-content .wy-alert-neutral.note strong, .rst-content .wy-alert-neutral.attention strong, .rst-content .wy-alert-neutral.caution strong, .rst-content .wy-alert-neutral.danger strong, .rst-content .wy-alert-neutral.error strong, .rst-content .wy-alert-neutral.hint strong, .rst-content .wy-alert-neutral.important strong, .rst-content .wy-alert-neutral.tip strong, .rst-content .wy-alert-neutral.warning strong {
    color: #404040;
}

.wy-alert.wy-alert-neutral a, .rst-content .wy-alert-neutral.note a, .rst-content .wy-alert-neutral.attention a, .rst-content .wy-alert-neutral.caution a, .rst-content .wy-alert-neutral.danger a, .rst-content .wy-alert-neutral.error a, .rst-content .wy-alert-neutral.hint a, .rst-content .wy-alert-neutral.important a, .rst-content .wy-alert-neutral.tip a, .rst-content .wy-alert-neutral.warning a {
    color: #2980b9;
}

.wy-tray-container {
    position: fixed;
    top: -50px;
    left: 0;
    width: 100%;
    -webkit-transition: top 0.2s ease-in;
    -moz-transition: top 0.2s ease-in;
    transition: top 0.2s ease-in;
}

.wy-tray-container.on {
    top: 0;
}

.wy-tray-container li {
    display: none;
    width: 100%;
    background: #343131;
    padding: 12px 24px;
    color: #fff;
    margin-bottom: 6px;
    text-align: center;
    box-shadow: 0 5px 5px 0 rgba(0, 0, 0, 0.1), 0px -1px 2px -1px rgba(255, 255, 255, 0.5) inset;
}

.wy-tray-container li.wy-tray-item-success {
    background: #27ae60;
}

.wy-tray-container li.wy-tray-item-info {
    background: #2980b9;
}

.wy-tray-container li.wy-tray-item-warning {
    background: #e67e22;
}

.wy-tray-container li.wy-tray-item-danger {
    background: #e74c3c;
}

.btn {
    display: inline-block;
    *display: inline;
    zoom: 1;
    line-height: normal;
    white-space: nowrap;
    vertical-align: baseline;
    text-align: center;
    cursor: pointer;
    -webkit-user-drag: none;
    -webkit-user-select: none;
    -moz-user-select: none;
    -ms-user-select: none;
    user-select: none;
    font-size: 100%;
    padding: 6px 12px;
    color: #fff;
    border: 1px solid rgba(0, 0, 0, 0.1);
    border-bottom: solid 3px rgba(0, 0, 0, 0.1);
    background-color: #27ae60;
    text-decoration: none;
    font-weight: 500;
    box-shadow: 0px 1px 2px -1px rgba(255, 255, 255, 0.5) inset;
    -webkit-transition: all 0.1s linear;
    -moz-transition: all 0.1s linear;
    transition: all 0.1s linear;
    outline-none: false;
}

.btn-hover {
    background: #2e8ece;
    color: #fff;
}

.btn:hover {
    background: #2cc36b;
    color: #fff;
}

.btn:focus {
    background: #2cc36b;
    color: #fff;
    outline: 0;
}

.btn:active {
    border-top: solid 3px rgba(0, 0, 0, 0.1);
    border-bottom: solid 1px rgba(0, 0, 0, 0.1);
    box-shadow: 0px 1px 2px -1px rgba(0, 0, 0, 0.5) inset;
}

.btn[disabled] {
    background-image: none;
    filter: progid:DXImageTransform.Microsoft.gradient(enabled = false);
    filter: alpha(opacity=40);
    opacity: 0.4;
    cursor: not-allowed;
    box-shadow: none;
}

.btn-disabled {
    background-image: none;
    filter: progid:DXImageTransform.Microsoft.gradient(enabled = false);
    filter: alpha(opacity=40);
    opacity: 0.4;
    cursor: not-allowed;
    box-shadow: none;
}

.btn-disabled:hover, .btn-disabled:focus, .btn-disabled:active {
    background-image: none;
    filter: progid:DXImageTransform.Microsoft.gradient(enabled = false);
    filter: alpha(opacity=40);
    opacity: 0.4;
    cursor: not-allowed;
    box-shadow: none;
}

.btn::-moz-focus-inner {
    padding: 0;
    border: 0;
}

.btn-small {
    font-size: 80%;
}

.btn-info {
    background-color: #2980b9 !important;
}

.btn-info:hover {
    background-color: #2e8ece !important;
}

.btn-neutral {
    background-color: #f3f6f6 !important;
    color: #404040 !important;
}

.btn-neutral:hover {
    background-color: #e5ebeb !important;
    color: #404040;
}

.btn-danger {
    background-color: #e74c3c !important;
}

.btn-danger:hover {
    background-color: #ea6153 !important;
}

.btn-warning {
    background-color: #e67e22 !important;
}

.btn-warning:hover {
    background-color: #e98b39 !important;
}

.btn-invert {
    background-color: #343131;
}

.btn-invert:hover {
    background-color: #413d3d !important;
}

.btn-link {
    background-color: transparent !important;
    color: #2980b9;
    border-color: transparent;
}

.btn-link:hover {
    background-color: transparent !important;
    color: #409ad5;
    border-color: transparent;
}

.btn-link:active {
    background-color: transparent !important;
    border-color: transparent;
    border-top: solid 1px transparent;
    border-bottom: solid 3px transparent;
}

.wy-btn-group .btn, .wy-control .btn {
    vertical-align: middle;
}

.wy-btn-group {
    margin-bottom: 24px;
    *zoom: 1;
}

.wy-btn-group:before, .wy-btn-group:after {
    display: table;
    content: "";
}

.wy-btn-group:after {
    clear: both;
}

.wy-dropdown {
    position: relative;
    display: inline-block;
}

.wy-dropdown:hover .wy-dropdown-menu {
    display: block;
}

.wy-dropdown .caret:after {
    font-family: fontawesome-webfont;
    content: "\f0d7";
    font-size: 70%;
}

.wy-dropdown-menu {
    position: absolute;
    top: 100%;
    left: 0;
    display: none;
    float: left;
    min-width: 100%;
    background: #fcfcfc;
    z-index: 100;
    border: solid 1px #cfd7dd;
    box-shadow: 0 5px 5px 0 rgba(0, 0, 0, 0.1);
    padding: 12px;
}

.wy-dropdown-menu>dd>a {
    display: block;
    clear: both;
    color: #404040;
    white-space: nowrap;
    font-size: 90%;
    padding: 0 12px;
}

.wy-dropdown-menu>dd>a:hover {
    background: #2980b9;
    color: #fff;
}

.wy-dropdown-menu>dd.divider {
    border-top: solid 1px #cfd7dd;
    margin: 6px 0;
}

.wy-dropdown-menu>dd.search {
    padding-bottom: 12px;
}

.wy-dropdown-menu>dd.search input[type="search"] {
    width: 100%;
}

.wy-dropdown-menu>dd.call-to-action {
    background: #e3e3e3;
    text-transform: uppercase;
    font-weight: 500;
    font-size: 80%;
}

.wy-dropdown-menu>dd.call-to-action:hover {
    background: #e3e3e3;
}

.wy-dropdown-menu>dd.call-to-action .btn {
    color: #fff;
}

.wy-dropdown.wy-dropdown-bubble .wy-dropdown-menu {
    background: #fcfcfc;
    margin-top: 2px;
}

.wy-dropdown.wy-dropdown-bubble .wy-dropdown-menu a {
    padding: 6px 12px;
}

.wy-dropdown.wy-dropdown-bubble .wy-dropdown-menu a:hover {
    background: #2980b9;
    color: #fff;
}

.wy-dropdown.wy-dropdown-left .wy-dropdown-menu {
    right: 0;
    text-align: right;
}

.wy-dropdown-arrow:before {
    content: " ";
    border-bottom: 5px solid #f5f5f5;
    border-left: 5px solid transparent;
    border-right: 5px solid transparent;
    position: absolute;
    display: block;
    top: -4px;
    left: 50%;
    margin-left: -3px;
}

.wy-dropdown-arrow.wy-dropdown-arrow-left:before {
    left: 11px;
}

.wy-form-stacked select {
    display: block;
}

.wy-form-aligned input, .wy-form-aligned textarea, .wy-form-aligned select, .wy-form-aligned .wy-help-inline, .wy-form-aligned label {
    display: inline-block;
    *display: inline;
    *zoom: 1;
    vertical-align: middle;
}

.wy-form-aligned .wy-control-group>label {
    display: inline-block;
    vertical-align: middle;
    width: 10em;
    margin: 0.5em 1em 0 0;
    float: left;
}

.wy-form-aligned .wy-control {
    float: left;
}

.wy-form-aligned .wy-control label {
    display: block;
}

.wy-form-aligned .wy-control select {
    margin-top: 0.5em;
}

fieldset {
    border: 0;
    margin: 0;
    padding: 0;
}

legend {
    display: block;
    width: 100%;
    border: 0;
    padding: 0;
    white-space: normal;
    margin-bottom: 24px;
    font-size: 150%;
    *margin-left: -7px;
}

label {
    display: block;
    margin: 0 0 0.3125em 0;
    color: #999;
    font-size: 90%;
}

button, input, select, textarea {
    font-size: 100%;
    margin: 0;
    vertical-align: baseline;
    *vertical-align: middle;
}

button, input {
    line-height: normal;
}

button {
    -webkit-appearance: button;
    cursor: pointer;
    *overflow: visible;
}

button::-moz-focus-inner, input::-moz-focus-inner {
    border: 0;
    padding: 0;
}

button[disabled] {
    cursor: default;
}

input[type="button"], input[type="reset"], input[type="submit"] {
    -webkit-appearance: button;
    cursor: pointer;
    *overflow: visible;
}

input[type="text"], input[type="password"], input[type="email"], input[type="url"], input[type="date"], input[type="month"], input[type="time"], input[type="datetime"], input[type="datetime-local"], input[type="week"], input[type="number"], input[type="search"], input[type="tel"], input[type="color"] {
    -webkit-appearance: none;
    padding: 6px;
    display: inline-block;
    border: 1px solid #ccc;
    font-size: 80%;
    font-family: "Lato", "proxima-nova", "Helvetica Neue", Arial, sans-serif;
    box-shadow: inset 0 1px 3px #ddd;
    border-radius: 0;
    -webkit-transition: border 0.3s linear;
    -moz-transition: border 0.3s linear;
    transition: border 0.3s linear;
}

input[type="datetime-local"] {
    padding: 0.34375em 0.625em;
}

input[disabled] {
    cursor: default;
}

input[type="checkbox"], input[type="radio"] {
    -webkit-box-sizing: border-box;
    -moz-box-sizing: border-box;
    box-sizing: border-box;
    padding: 0;
    margin-right: 0.3125em;
    *height: 13px;
    *width: 13px;
}

input[type="search"] {
    -webkit-box-sizing: border-box;
    -moz-box-sizing: border-box;
    box-sizing: border-box;
}

input[type="search"]::-webkit-search-cancel-button, input[type="search"]::-webkit-search-decoration {
    -webkit-appearance: none;
}

input[type="text"]:focus, input[type="password"]:focus, input[type="email"]:focus, input[type="url"]:focus, input[type="date"]:focus, input[type="month"]:focus, input[type="time"]:focus, input[type="datetime"]:focus, input[type="datetime-local"]:focus, input[type="week"]:focus, input[type="number"]:focus, input[type="search"]:focus, input[type="tel"]:focus, input[type="color"]:focus {
    outline: 0;
    outline: thin dotted \9;
    border-color: #2980b9;
}

input.no-focus:focus {
    border-color: #ccc !important;
}

input[type="file"]:focus, input[type="radio"]:focus, input[type="checkbox"]:focus {
    outline: thin dotted #333;
    outline: 1px auto #129fea;
}

input[type="text"][disabled], input[type="password"][disabled], input[type="email"][disabled], input[type="url"][disabled], input[type="date"][disabled], input[type="month"][disabled], input[type="time"][disabled], input[type="datetime"][disabled], input[type="datetime-local"][disabled], input[type="week"][disabled], input[type="number"][disabled], input[type="search"][disabled], input[type="tel"][disabled], input[type="color"][disabled] {
    cursor: not-allowed;
    background-color: #f3f6f6;
    color: #cad2d3;
}

input:focus:invalid, textarea:focus:invalid, select:focus:invalid {
    color: #e74c3c;
    border: 1px solid #e74c3c;
}

input:focus:invalid:focus, textarea:focus:invalid:focus, select:focus:invalid:focus {
    border-color: #e9322d;
}

input[type="file"]:focus:invalid:focus, input[type="radio"]:focus:invalid:focus, input[type="checkbox"]:focus:invalid:focus {
    outline-color: #e9322d;
}

input.wy-input-large {
    padding: 12px;
    font-size: 100%;
}

textarea {
    overflow: auto;
    vertical-align: top;
    width: 100%;
}

select, textarea {
    padding: 0.5em 0.625em;
    display: inline-block;
    border: 1px solid #ccc;
    font-size: 0.8em;
    box-shadow: inset 0 1px 3px #ddd;
    -webkit-transition: border 0.3s linear;
    -moz-transition: border 0.3s linear;
    transition: border 0.3s linear;
}

select {
    border: 1px solid #ccc;
    background-color: #fff;
}

select[multiple] {
    height: auto;
}

select:focus, textarea:focus {
    outline: 0;
}

select[disabled], textarea[disabled], input[readonly], select[readonly], textarea[readonly] {
    cursor: not-allowed;
    background-color: #fff;
    color: #cad2d3;
    border-color: transparent;
}

.wy-checkbox, .wy-radio {
    margin: 0.5em 0;
    color: #404040 !important;
    display: block;
}

.wy-form-message-inline {
    display: inline-block;
    *display: inline;
    *zoom: 1;
    vertical-align: middle;
}

.wy-input-prefix, .wy-input-suffix {
    white-space: nowrap;
}

.wy-input-prefix .wy-input-context, .wy-input-suffix .wy-input-context {
    padding: 6px;
    display: inline-block;
    font-size: 80%;
    background-color: #f3f6f6;
    border: solid 1px #ccc;
    color: #999;
}

.wy-input-suffix .wy-input-context {
    border-left: 0;
}

.wy-input-prefix .wy-input-context {
    border-right: 0;
}

.wy-inline-validate {
    white-space: nowrap;
}

.wy-inline-validate .wy-input-context {
    padding: 0.5em 0.625em;
    display: inline-block;
    font-size: 80%;
}

.wy-inline-validate.wy-inline-validate-success .wy-input-context {
    color: #27ae60;
}

.wy-inline-validate.wy-inline-validate-danger .wy-input-context {
    color: #e74c3c;
}

.wy-inline-validate.wy-inline-validate-warning .wy-input-context {
    color: #e67e22;
}

.wy-inline-validate.wy-inline-validate-info .wy-input-context {
    color: #2980b9;
}

.wy-control-group {
    margin-bottom: 24px;
    *zoom: 1;
}

.wy-control-group:before, .wy-control-group:after {
    display: table;
    content: "";
}

.wy-control-group:after {
    clear: both;
}

.wy-control-group.wy-control-group-error .wy-form-message, .wy-control-group.wy-control-group-error label {
    color: #e74c3c;
}

.wy-control-group.wy-control-group-error input[type="text"], .wy-control-group.wy-control-group-error input[type="password"], .wy-control-group.wy-control-group-error input[type="email"], .wy-control-group.wy-control-group-error input[type="url"], .wy-control-group.wy-control-group-error input[type="date"], .wy-control-group.wy-control-group-error input[type="month"], .wy-control-group.wy-control-group-error input[type="time"], .wy-control-group.wy-control-group-error input[type="datetime"], .wy-control-group.wy-control-group-error input[type="datetime-local"], .wy-control-group.wy-control-group-error input[type="week"], .wy-control-group.wy-control-group-error input[type="number"], .wy-control-group.wy-control-group-error input[type="search"], .wy-control-group.wy-control-group-error input[type="tel"], .wy-control-group.wy-control-group-error input[type="color"] {
    border: solid 2px #e74c3c;
}

.wy-control-group.wy-control-group-error textarea {
    border: solid 2px #e74c3c;
}

.wy-control-group.fluid-input input[type="text"], .wy-control-group.fluid-input input[type="password"], .wy-control-group.fluid-input input[type="email"], .wy-control-group.fluid-input input[type="url"], .wy-control-group.fluid-input input[type="date"], .wy-control-group.fluid-input input[type="month"], .wy-control-group.fluid-input input[type="time"], .wy-control-group.fluid-input input[type="datetime"], .wy-control-group.fluid-input input[type="datetime-local"], .wy-control-group.fluid-input input[type="week"], .wy-control-group.fluid-input input[type="number"], .wy-control-group.fluid-input input[type="search"], .wy-control-group.fluid-input input[type="tel"], .wy-control-group.fluid-input input[type="color"] {
    width: 100%;
}

.wy-form-message-inline {
    display: inline-block;
    padding-left: 0.3em;
    color: #666;
    vertical-align: middle;
    font-size: 90%;
}

.wy-form-message {
    display: block;
    color: #ccc;
    font-size: 70%;
    margin-top: 0.3125em;
    font-style: italic;
}

.wy-tag-input-group {
    padding: 4px 4px 0px 4px;
    display: inline-block;
    border: 1px solid #ccc;
    font-size: 80%;
    font-family: "Lato", "proxima-nova", "Helvetica Neue", Arial, sans-serif;
    box-shadow: inset 0 1px 3px #ddd;
    -webkit-transition: border 0.3s linear;
    -moz-transition: border 0.3s linear;
    transition: border 0.3s linear;
}

.wy-tag-input-group .wy-tag {
    display: inline-block;
    background-color: rgba(0, 0, 0, 0.1);
    padding: 0.5em 0.625em;
    border-radius: 2px;
    position: relative;
    margin-bottom: 4px;
}

.wy-tag-input-group .wy-tag .wy-tag-remove {
    color: #ccc;
    margin-left: 5px;
}

.wy-tag-input-group .wy-tag .wy-tag-remove:hover {
    color: #e74c3c;
}

.wy-tag-input-group label {
    margin-left: 5px;
    display: inline-block;
    margin-bottom: 0;
}

.wy-tag-input-group input {
    border: none;
    font-size: 100%;
    margin-bottom: 4px;
    box-shadow: none;
}

.wy-form-upload {
    border: solid 1px #ccc;
    border-bottom: solid 3px #ccc;
    background-color: #fff;
    padding: 24px;
    display: inline-block;
    text-align: center;
    cursor: pointer;
    color: #404040;
    -webkit-transition: border-color 0.1s ease-in;
    -moz-transition: border-color 0.1s ease-in;
    transition: border-color 0.1s ease-in;
    *zoom: 1;
}

.wy-form-upload:before, .wy-form-upload:after {
    display: table;
    content: "";
}

.wy-form-upload:after {
    clear: both;
}

@media screen and (max-width: 480px) {
    .wy-form-upload {
        width: 100%;
    }
}

.wy-form-upload .image-drop {
    display: none;
}

.wy-form-upload .image-desktop {
    display: none;
}

.wy-form-upload .image-loading {
    display: none;
}

.wy-form-upload .wy-form-upload-icon {
    display: block;
    font-size: 32px;
    color: #b3b3b3;
}

.wy-form-upload .image-drop .wy-form-upload-icon {
    color: #27ae60;
}

.wy-form-upload p {
    font-size: 90%;
}

.wy-form-upload .wy-form-upload-image {
    float: left;
    margin-right: 24px;
}

@media screen and (max-width: 480px) {
    .wy-form-upload .wy-form-upload-image {
        width: 100%;
        margin-bottom: 24px;
    }
}

.wy-form-upload img {
    max-width: 125px;
    max-height: 125px;
    opacity: 0.9;
    -webkit-transition: opacity 0.1s ease-in;
    -moz-transition: opacity 0.1s ease-in;
    transition: opacity 0.1s ease-in;
}

.wy-form-upload .wy-form-upload-content {
    float: left;
}

@media screen and (max-width: 480px) {
    .wy-form-upload .wy-form-upload-content {
        width: 100%;
    }
}

.wy-form-upload:hover {
    border-color: #b3b3b3;
    color: #404040;
}

.wy-form-upload:hover .image-desktop {
    display: block;
}

.wy-form-upload:hover .image-drag {
    display: none;
}

.wy-form-upload:hover img {
    opacity: 1;
}

.wy-form-upload:active {
    border-top: solid 3px #ccc;
    border-bottom: solid 1px #ccc;
}

.wy-form-upload.wy-form-upload-big {
    width: 100%;
    text-align: center;
    padding: 72px;
}

.wy-form-upload.wy-form-upload-big .wy-form-upload-content {
    float: none;
}

.wy-form-upload.wy-form-upload-file p {
    margin-bottom: 0;
}

.wy-form-upload.wy-form-upload-file .wy-form-upload-icon {
    display: inline-block;
    font-size: inherit;
}

.wy-form-upload.wy-form-upload-drop {
    background-color: #ddf7e8;
}

.wy-form-upload.wy-form-upload-drop .image-drop {
    display: block;
}

.wy-form-upload.wy-form-upload-drop .image-desktop {
    display: none;
}

.wy-form-upload.wy-form-upload-drop .image-drag {
    display: none;
}

.wy-form-upload.wy-form-upload-loading .image-drag {
    display: none;
}

.wy-form-upload.wy-form-upload-loading .image-desktop {
    display: none;
}

.wy-form-upload.wy-form-upload-loading .image-loading {
    display: block;
}

.wy-form-upload.wy-form-upload-loading .wy-input-prefix {
    display: none;
}

.wy-form-upload.wy-form-upload-loading p {
    margin-bottom: 0;
}

.rotate-90 {
    -webkit-transform: rotate(90deg);
    -moz-transform: rotate(90deg);
    -ms-transform: rotate(90deg);
    -o-transform: rotate(90deg);
    transform: rotate(90deg);
}

.rotate-180 {
    -webkit-transform: rotate(180deg);
    -moz-transform: rotate(180deg);
    -ms-transform: rotate(180deg);
    -o-transform: rotate(180deg);
    transform: rotate(180deg);
}

.rotate-270 {
    -webkit-transform: rotate(270deg);
    -moz-transform: rotate(270deg);
    -ms-transform: rotate(270deg);
    -o-transform: rotate(270deg);
    transform: rotate(270deg);
}

.mirror {
    -webkit-transform: scaleX(-1);
    -moz-transform: scaleX(-1);
    -ms-transform: scaleX(-1);
    -o-transform: scaleX(-1);
    transform: scaleX(-1);
}

.mirror.rotate-90 {
    -webkit-transform: scaleX(-1) rotate(90deg);
    -moz-transform: scaleX(-1) rotate(90deg);
    -ms-transform: scaleX(-1) rotate(90deg);
    -o-transform: scaleX(-1) rotate(90deg);
    transform: scaleX(-1) rotate(90deg);
}

.mirror.rotate-180 {
    -webkit-transform: scaleX(-1) rotate(180deg);
    -moz-transform: scaleX(-1) rotate(180deg);
    -ms-transform: scaleX(-1) rotate(180deg);
    -o-transform: scaleX(-1) rotate(180deg);
    transform: scaleX(-1) rotate(180deg);
}

.mirror.rotate-270 {
    -webkit-transform: scaleX(-1) rotate(270deg);
    -moz-transform: scaleX(-1) rotate(270deg);
    -ms-transform: scaleX(-1) rotate(270deg);
    -o-transform: scaleX(-1) rotate(270deg);
    transform: scaleX(-1) rotate(270deg);
}

.wy-form-gallery-manage {
    margin-left: -12px;
    margin-right: -12px;
}

.wy-form-gallery-manage li {
    float: left;
    padding: 12px;
    width: 20%;
    cursor: pointer;
}

@media screen and (max-width: 768px) {
    .wy-form-gallery-manage li {
        width: 25%;
    }
}

@media screen and (max-width: 480px) {
    .wy-form-gallery-manage li {
        width: 50%;
    }
}

.wy-form-gallery-manage li:active {
    cursor: move;
}

.wy-form-gallery-manage li>a {
    padding: 12px;
    background-color: #fff;
    border: solid 1px #e1e4e5;
    border-bottom: solid 3px #e1e4e5;
    display: inline-block;
    -webkit-transition: all 0.1s ease-in;
    -moz-transition: all 0.1s ease-in;
    transition: all 0.1s ease-in;
}

.wy-form-gallery-manage li>a:active {
    border: solid 1px #ccc;
    border-top: solid 3px #ccc;
}

.wy-form-gallery-manage img {
    width: 100%;
    -webkit-transition: all 0.05s ease-in;
    -moz-transition: all 0.05s ease-in;
    transition: all 0.05s ease-in;
}

li.wy-form-gallery-edit {
    position: relative;
    color: #fff;
    padding: 24px;
    width: 100%;
    display: block;
    background-color: #343131;
    border-radius: 4px;
}

li.wy-form-gallery-edit .arrow {
    position: absolute;
    display: block;
    top: -50px;
    left: 50%;
    margin-left: -25px;
    z-index: 500;
    height: 0;
    width: 0;
    border-color: transparent;
    border-style: solid;
    border-width: 25px;
    border-bottom-color: #343131;
}

@media only screen and (max-width: 480px) {
    .wy-form button[type="submit"] {
        margin: 0.7em 0 0;
    }

    .wy-form input[type="text"], .wy-form input[type="password"], .wy-form input[type="email"], .wy-form input[type="url"], .wy-form input[type="date"], .wy-form input[type="month"], .wy-form input[type="time"], .wy-form input[type="datetime"], .wy-form input[type="datetime-local"], .wy-form input[type="week"], .wy-form input[type="number"], .wy-form input[type="search"], .wy-form input[type="tel"], .wy-form input[type="color"] {
        margin-bottom: 0.3em;
        display: block;
    }

    .wy-form label {
        margin-bottom: 0.3em;
        display: block;
    }

    .wy-form input[type="password"], .wy-form input[type="email"], .wy-form input[type="url"], .wy-form input[type="date"], .wy-form input[type="month"], .wy-form input[type="time"], .wy-form input[type="datetime"], .wy-form input[type="datetime-local"], .wy-form input[type="week"], .wy-form input[type="number"], .wy-form input[type="search"], .wy-form input[type="tel"], .wy-form input[type="color"] {
        margin-bottom: 0;
    }

    .wy-form-aligned .wy-control-group label {
        margin-bottom: 0.3em;
        text-align: left;
        display: block;
        width: 100%;
    }

    .wy-form-aligned .wy-controls {
        margin: 1.5em 0 0 0;
    }

    .wy-form .wy-help-inline, .wy-form-message-inline, .wy-form-message {
        display: block;
        font-size: 80%;
        padding: 0.2em 0 0.8em;
    }
}

@media screen and (max-width: 768px) {
    .tablet-hide {
        display: none;
    }
}

@media screen and (max-width: 480px) {
    .mobile-hide {
        display: none;
    }
}

.float-left {
    float: left;
}

.float-right {
    float: right;
}

.full-width {
    width: 100%;
}

.wy-grid-one-col {
    *zoom: 1;
    max-width: 68em;
    margin-left: auto;
    margin-right: auto;
    max-width: 1066px;
    margin-top: 1.618em;
}

.wy-grid-one-col:before, .wy-grid-one-col:after {
    display: table;
    content: "";
}

.wy-grid-one-col:after {
    clear: both;
}

.wy-grid-one-col section {
    display: block;
    float: left;
    margin-right: 2.35765%;
    width: 100%;
    background: #fff;
    padding: 1.618em;
    margin-right: 0;
}

.wy-grid-one-col section:last-child {
    margin-right: 0;
}

.wy-grid-index-card {
    *zoom: 1;
    max-width: 68em;
    margin-left: auto;
    margin-right: auto;
    max-width: 460px;
    margin-top: 1.618em;
    background: #fff;
    padding: 1.618em;
}

.wy-grid-index-card:before, .wy-grid-index-card:after {
    display: table;
    content: "";
}

.wy-grid-index-card:after {
    clear: both;
}

.wy-grid-index-card header, .wy-grid-index-card section, .wy-grid-index-card aside {
    display: block;
    float: left;
    margin-right: 2.35765%;
    width: 100%;
}

.wy-grid-index-card header:last-child, .wy-grid-index-card section:last-child, .wy-grid-index-card aside:last-child {
    margin-right: 0;
}

.wy-grid-index-card.twocol {
    max-width: 768px;
}

.wy-grid-index-card.twocol section {
    display: block;
    float: left;
    margin-right: 2.35765%;
    width: 48.82117%;
}

.wy-grid-index-card.twocol section:last-child {
    margin-right: 0;
}

.wy-grid-index-card.twocol aside {
    display: block;
    float: left;
    margin-right: 2.35765%;
    width: 48.82117%;
}

.wy-grid-index-card.twocol aside:last-child {
    margin-right: 0;
}

.wy-grid-search-filter {
    *zoom: 1;
    max-width: 68em;
    margin-left: auto;
    margin-right: auto;
    margin-bottom: 24px;
}

.wy-grid-search-filter:before, .wy-grid-search-filter:after {
    display: table;
    content: "";
}

.wy-grid-search-filter:after {
    clear: both;
}

.wy-grid-search-filter .wy-grid-search-filter-input {
    display: block;
    float: left;
    margin-right: 2.35765%;
    width: 74.41059%;
}

.wy-grid-search-filter .wy-grid-search-filter-input:last-child {
    margin-right: 0;
}

.wy-grid-search-filter .wy-grid-search-filter-btn {
    display: block;
    float: left;
    margin-right: 2.35765%;
    width: 23.23176%;
}

.wy-grid-search-filter .wy-grid-search-filter-btn:last-child {
    margin-right: 0;
}

.wy-table, .rst-content table.docutils, .rst-content table.field-list {
    border-collapse: collapse;
    border-spacing: 0;
    empty-cells: show;
    margin-bottom: 24px;
}

.wy-table caption, .rst-content table.docutils caption, .rst-content table.field-list caption {
    color: #fff;
    font: italic 85%/1 arial, sans-serif;
    padding: 1em 0;
    text-align: center;
}

.wy-table td, .rst-content table.docutils td, .rst-content table.field-list td, .wy-table th, .rst-content table.docutils th, .rst-content table.field-list th {
    font-size: 90%;
    margin: 0;
    overflow: visible;
    padding: 8px 16px;
}

.wy-table td:first-child, .rst-content table.docutils td:first-child, .rst-content table.field-list td:first-child, .wy-table th:first-child, .rst-content table.docutils th:first-child, .rst-content table.field-list th:first-child {
    border-left-width: 0;
}

.wy-table thead, .rst-content table.docutils thead, .rst-content table.field-list thead {
    color: #000;
    text-align: left;
    vertical-align: bottom;
    white-space: nowrap;
}

.wy-table thead th, .rst-content table.docutils thead th, .rst-content table.field-list thead th {
    font-weight: bold;
    border-bottom: solid 2px #e1e4e5;
}

.wy-table td, .rst-content table.docutils td, .rst-content table.field-list td {
    background-color: transparent;
    vertical-align: middle;
}

.wy-table td p, .rst-content table.docutils td p, .rst-content table.field-list td p {
    line-height: 18px;
    margin-bottom: 0;
}

.wy-table .wy-table-cell-min, .rst-content table.docutils .wy-table-cell-min, .rst-content table.field-list .wy-table-cell-min {
    width: 1%;
    padding-right: 0;
}

.wy-table .wy-table-cell-min input[type=checkbox], .rst-content table.docutils .wy-table-cell-min input[type=checkbox], .rst-content table.field-list .wy-table-cell-min input[type=checkbox], .wy-table .wy-table-cell-min input[type=checkbox], .rst-content table.docutils .wy-table-cell-min input[type=checkbox], .rst-content table.field-list .wy-table-cell-min input[type=checkbox] {
    margin: 0;
}

.wy-table-secondary {
    color: gray;
    font-size: 90%;
}

.wy-table-tertiary {
    color: gray;
    font-size: 80%;
}

.wy-table-odd td, .wy-table-striped tr:nth-child(2n-1) td, .rst-content table.docutils:not(.field-list) tr:nth-child(2n-1) td {
    background-color: #f3f6f6;
}

.wy-table-backed {
    background-color: #f3f6f6;
}

.wy-table-bordered-all, .rst-content table.docutils {
    border: 1px solid #e1e4e5;
}

.wy-table-bordered-all td, .rst-content table.docutils td {
    border-bottom: 1px solid #e1e4e5;
    border-left: 1px solid #e1e4e5;
}

.wy-table-bordered-all tbody>tr:last-child td, .rst-content table.docutils tbody>tr:last-child td {
    border-bottom-width: 0;
}

.wy-table-bordered {
    border: 1px solid #e1e4e5;
}

.wy-table-bordered-rows td {
    border-bottom: 1px solid #e1e4e5;
}

.wy-table-bordered-rows tbody>tr:last-child td {
    border-bottom-width: 0;
}

.wy-table-horizontal tbody>tr:last-child td {
    border-bottom-width: 0;
}

.wy-table-horizontal td, .wy-table-horizontal th {
    border-width: 0 0 1px 0;
    border-bottom: 1px solid #e1e4e5;
}

.wy-table-horizontal tbody>tr:last-child td {
    border-bottom-width: 0;
}

.wy-table-responsive {
    margin-bottom: 24px;
    max-width: 100%;
    overflow: auto;
}

.wy-table-responsive table {
    margin-bottom: 0 !important;
}

.wy-table-responsive table td, .wy-table-responsive table th {
    white-space: nowrap;
}

html {
    height: 100%;
    overflow-x: hidden;
}

body {
    font-family: "Lato", "proxima-nova", "Helvetica Neue", Arial, sans-serif;
    font-weight: normal;
    color: #404040;
    min-height: 100%;
    overflow-x: hidden;
    background: #edf0f2;
}

a {
    color: #2980b9;
    text-decoration: none;
}

a:hover {
    color: #3091d1;
}

.link-danger {
    color: #e74c3c;
}

.link-danger:hover {
    color: #d62c1a;
}

.text-left {
    text-align: left;
}

.text-center {
    text-align: center;
}

.text-right {
    text-align: right;
}

h1, h2, h3, h4, h5, h6, legend {
    margin-top: 0;
    font-weight: 700;
    font-family: "Roboto Slab", "ff-tisa-web-pro", "Georgia", Arial, sans-serif;
}

p {
    line-height: 24px;
    margin: 0;
    font-size: 16px;
    margin-bottom: 24px;
}

h1 {
    font-size: 175%;
}

h2 {
    font-size: 150%;
}

h3 {
    font-size: 125%;
}

h4 {
    font-size: 115%;
}

h5 {
    font-size: 110%;
}

h6 {
    font-size: 100%;
}

small {
    font-size: 80%;
}

code, .rst-content tt {
    white-space: nowrap;
    max-width: 100%;
    background: #fff;
    border: solid 1px #e1e4e5;
    font-size: 75%;
    padding: 0 5px;
    font-family: "Inconsolata", "Consolata", "Monaco", monospace;
    color: #e74c3c;
    overflow-x: auto;
}

code.code-large, .rst-content tt.code-large {
    font-size: 90%;
}

.full-width {
    width: 100%;
}

.wy-plain-list-disc, .rst-content .section ul, .rst-content .toctree-wrapper ul {
    list-style: disc;
    line-height: 24px;
    margin-bottom: 24px;
}

.wy-plain-list-disc li, .rst-content .section ul li, .rst-content .toctree-wrapper ul li {
    list-style: disc;
    margin-left: 24px;
}

.wy-plain-list-disc li ul, .rst-content .section ul li ul, .rst-content .toctree-wrapper ul li ul {
    margin-bottom: 0;
}

.wy-plain-list-disc li li, .rst-content .section ul li li, .rst-content .toctree-wrapper ul li li {
    list-style: circle;
}

.wy-plain-list-disc li li li, .rst-content .section ul li li li, .rst-content .toctree-wrapper ul li li li {
    list-style: square;
}

.wy-plain-list-decimal, .rst-content .section ol, .rst-content ol.arabic {
    list-style: decimal;
    line-height: 24px;
    margin-bottom: 24px;
}

.wy-plain-list-decimal li, .rst-content .section ol li, .rst-content ol.arabic li {
    list-style: decimal;
    margin-left: 24px;
}

.wy-type-large {
    font-size: 120%;
}

.wy-type-normal {
    font-size: 100%;
}

.wy-type-small {
    font-size: 100%;
}

.wy-type-strike {
    text-decoration: line-through;
}

.wy-text-warning {
    color: #e67e22 !important;
}

a.wy-text-warning:hover {
    color: #eb9950 !important;
}

.wy-text-info {
    color: #2980b9 !important;
}

a.wy-text-info:hover {
    color: #409ad5 !important;
}

.wy-text-success {
    color: #27ae60 !important;
}

a.wy-text-success:hover {
    color: #36d278 !important;
}

.wy-text-danger {
    color: #e74c3c !important;
}

a.wy-text-danger:hover {
    color: #ed7669 !important;
}

.wy-text-neutral {
    color: #404040 !important;
}

a.wy-text-neutral:hover {
    color: #595959 !important;
}

.codeblock-example {
    border: 1px solid #e1e4e5;
    border-bottom: none;
    padding: 24px;
    padding-top: 48px;
    font-weight: 500;
    background: #fff;
    position: relative;
}

.codeblock-example:after {
    content: "Example";
    position: absolute;
    top: 0px;
    left: 0px;
    background: #9b59b6;
    color: #fff;
    padding: 6px 12px;
}

.codeblock-example.prettyprint-example-only {
    border: 1px solid #e1e4e5;
    margin-bottom: 24px;
}

.codeblock, div[class^='highlight'] {
    border: 1px solid #e1e4e5;
    padding: 0px;
    overflow-x: auto;
    background: #fff;
    margin: 1px 0 24px 0;
}

.codeblock div[class^='highlight'], div[class^='highlight'] div[class^='highlight'] {
    border: none;
    background: none;
    margin: 0;
}

.linenodiv pre {
    border-right: solid 1px #e6e9ea;
    margin: 0;
    padding: 12px 12px;
    font-family: "Inconsolata", "Consolata", "Monaco", monospace;
    font-size: 12px;
    line-height: 1.5;
    color: #d9d9d9;
}

div[class^='highlight'] pre {
    white-space: pre;
    margin: 0;
    padding: 12px 12px;
    font-family: "Inconsolata", "Consolata", "Monaco", monospace;
    font-size: 12px;
    line-height: 1.5;
    display: block;
    overflow: auto;
    color: #404040;
}

pre.literal-block {
    @extends .codeblock;
}

@media print {
    .codeblock, div[class^='highlight'], div[class^='highlight'] pre {
        white-space: pre-wrap;
    }
}

.hll {
    background-color: #f8f8f8;
    border: 1px solid #ccc;
    padding: 1.5px 5px;
}

.c {
    color: #998;
    font-style: italic;
}

.err {
    color: #a61717;
    background-color: #e3d2d2;
}

.k {
    font-weight: bold;
}

.o {
    font-weight: bold;
}

.cm {
    color: #998;
    font-style: italic;
}

.cp {
    color: #999;
    font-weight: bold;
}

.c1 {
    color: #998;
    font-style: italic;
}

.cs {
    color: #999;
    font-weight: bold;
    font-style: italic;
}

.gd {
    color: #000;
    background-color: #fdd;
}

.gd .x {
    color: #000;
    background-color: #faa;
}

.ge {
    font-style: italic;
}

.gr {
    color: #a00;
}

.gh {
    color: #999;
}

.gi {
    color: #000;
    background-color: #dfd;
}

.gi .x {
    color: #000;
    background-color: #afa;
}

.go {
    color: #888;
}

.gp {
    color: #555;
}

.gs {
    font-weight: bold;
}

.gu {
    color: purple;
    font-weight: bold;
}

.gt {
    color: #a00;
}

.kc {
    font-weight: bold;
}

.kd {
    font-weight: bold;
}

.kn {
    font-weight: bold;
}

.kp {
    font-weight: bold;
}

.kr {
    font-weight: bold;
}

.kt {
    color: #458;
    font-weight: bold;
}

.m {
    color: #099;
}

.s {
    color: #d14;
}

.n {
    color: #333;
}

.na {
    color: teal;
}

.nb {
    color: #0086b3;
}

.nc {
    color: #458;
    font-weight: bold;
}

.no {
    color: teal;
}

.ni {
    color: purple;
}

.ne {
    color: #900;
    font-weight: bold;
}

.nf {
    color: #900;
    font-weight: bold;
}

.nn {
    color: #555;
}

.nt {
    color: navy;
}

.nv {
    color: teal;
}

.ow {
    font-weight: bold;
}

.w {
    color: #bbb;
}

.mf {
    color: #099;
}

.mh {
    color: #099;
}

.mi {
    color: #099;
}

.mo {
    color: #099;
}

.sb {
    color: #d14;
}

.sc {
    color: #d14;
}

.sd {
    color: #d14;
}

.s2 {
    color: #d14;
}

.se {
    color: #d14;
}

.sh {
    color: #d14;
}

.si {
    color: #d14;
}

.sx {
    color: #d14;
}

.sr {
    color: #009926;
}

.s1 {
    color: #d14;
}

.ss {
    color: #990073;
}

.bp {
    color: #999;
}

.vc {
    color: teal;
}

.vg {
    color: teal;
}

.vi {
    color: teal;
}

.il {
    color: #099;
}

.gc {
    color: #999;
    background-color: #eaf2f5;
}

.wy-breadcrumbs li {
    display: inline-block;
}

.wy-breadcrumbs li.wy-breadcrumbs-aside {
    float: right;
}

.wy-breadcrumbs li a {
    display: inline-block;
    padding: 5px;
}

.wy-breadcrumbs li a:first-child {
    padding-left: 0;
}

.wy-breadcrumbs-extra {
    margin-bottom: 0;
    color: #b3b3b3;
    font-size: 80%;
    display: inline-block;
}

@media screen and (max-width: 480px) {
    .wy-breadcrumbs-extra {
        display: none;
    }

    .wy-breadcrumbs li.wy-breadcrumbs-aside {
        display: none;
    }
}

@media print {
    .wy-breadcrumbs li.wy-breadcrumbs-aside {
        display: none;
    }
}

.wy-affix {
    position: fixed;
    top: 1.618em;
}

.wy-menu a:hover {
    text-decoration: none;
}

.wy-menu-horiz {
    *zoom: 1;
}

.wy-menu-horiz:before, .wy-menu-horiz:after {
    display: table;
    content: "";
}

.wy-menu-horiz:after {
    clear: both;
}

.wy-menu-horiz ul, .wy-menu-horiz li {
    display: inline-block;
}

.wy-menu-horiz li:hover {
    background: rgba(255, 255, 255, 0.1);
}

.wy-menu-horiz li.divide-left {
    border-left: solid 1px #404040;
}

.wy-menu-horiz li.divide-right {
    border-right: solid 1px #404040;
}

.wy-menu-horiz a {
    height: 32px;
    display: inline-block;
    line-height: 32px;
    padding: 0 16px;
}

.wy-menu-vertical header {
    height: 32px;
    display: inline-block;
    line-height: 32px;
    padding: 0 1.618em;
    display: block;
    font-weight: bold;
    text-transform: uppercase;
    font-size: 80%;
    color: #2980b9;
    white-space: nowrap;
}

.wy-menu-vertical ul {
    margin-bottom: 0;
}

.wy-menu-vertical li.divide-top {
    border-top: solid 1px #404040;
}

.wy-menu-vertical li.divide-bottom {
    border-bottom: solid 1px #404040;
}

.wy-menu-vertical li.current {
    background: #e3e3e3;
}

.wy-menu-vertical li.current a {
    color: gray;
    border-right: solid 1px #c9c9c9;
    padding: 0.4045em 2.427em;
}

.wy-menu-vertical li.current a:hover {
    background: #d6d6d6;
}

.wy-menu-vertical li.on a, .wy-menu-vertical li.current>a {
    color: #404040;
    padding: 0.4045em 1.618em;
    font-weight: bold;
    position: relative;
    background: #fcfcfc;
    border: none;
    border-bottom: solid 1px #c9c9c9;
    border-top: solid 1px #c9c9c9;
    padding-left: 1.618em -4px;
}

.wy-menu-vertical li.on a:hover, .wy-menu-vertical li.current>a:hover {
    background: #fcfcfc;
}

.wy-menu-vertical li.tocktree-l2.current>a {
    background: #c9c9c9;
}

.wy-menu-vertical li.current ul {
    display: block;
}

.wy-menu-vertical li ul {
    margin-bottom: 0;
    display: none;
}

.wy-menu-vertical li ul li a {
    margin-bottom: 0;
    color: #b3b3b3;
    font-weight: normal;
}

.wy-menu-vertical a {
    display: inline-block;
    line-height: 18px;
    padding: 0.4045em 1.618em;
    display: block;
    position: relative;
    font-size: 90%;
    color: #b3b3b3;
}

.wy-menu-vertical a:hover {
    background-color: #4e4a4a;
    cursor: pointer;
}

.wy-menu-vertical a:active {
    background-color: #2980b9;
    cursor: pointer;
    color: #fff;
}

.wy-side-nav-search {
    z-index: 200;
    background-color: #2980b9;
    text-align: center;
    padding: 0.809em;
    display: block;
    color: #fcfcfc;
    margin-bottom: 0.809em;
}

.wy-side-nav-search input[type=text] {
    width: 100%;
    border-radius: 50px;
    padding: 6px 12px;
    border-color: #2472a4;
}

.wy-side-nav-search img {
    display: block;
    margin: auto auto 0.809em auto;
    height: 214px;
    width: 26px;
    background-color: #2980b9;
    padding: 5px;
/*  border-radius: 100%; */
}

.wy-side-nav-search>a, .wy-side-nav-search .wy-dropdown>a {
    color: #fcfcfc;
    font-size: 100%;
    font-weight: bold;
    display: inline-block;
    padding: 4px 6px;
    margin-bottom: 0.809em;
}

.wy-side-nav-search>a:hover, .wy-side-nav-search .wy-dropdown>a:hover {
    background: rgba(255, 255, 255, 0.1);
}

.wy-nav .wy-menu-vertical header {
    color: #2980b9;
}

.wy-nav .wy-menu-vertical a {
    color: #b3b3b3;
}

.wy-nav .wy-menu-vertical a:hover {
    background-color: #2980b9;
    color: #fff;
}

[data-menu-wrap] {
    -webkit-transition: all 0.2s ease-in;
    -moz-transition: all 0.2s ease-in;
    transition: all 0.2s ease-in;
    position: absolute;
    opacity: 1;
    width: 100%;
    opacity: 0;
}

[data-menu-wrap].move-center {
    left: 0;
    right: auto;
    opacity: 1;
}

[data-menu-wrap].move-left {
    right: auto;
    left: -100%;
    opacity: 0;
}

[data-menu-wrap].move-right {
    right: -100%;
    left: auto;
    opacity: 0;
}

.wy-body-for-nav {
    background: left repeat-y #fff;
    background-image: url(data:image/png;
    base64, iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAIAAACQd1PeAAAAGXRFWHRTb2Z0d2FyZQBBZG9iZSBJbWFnZVJlYWR5ccllPAAAAyRpVFh0WE1MOmNvbS5hZG9iZS54bXAAAAAAADw/eHBhY2tldCBiZWdpbj0i77u/IiBpZD0iVzVNME1wQ2VoaUh6cmVTek5UY3prYzlkIj8+IDx4OnhtcG1ldGEgeG1sbnM6eD0iYWRvYmU6bnM6bWV0YS8iIHg6eG1wdGs9IkFkb2JlIFhNUCBDb3JlIDUuMy1jMDExIDY2LjE0NTY2MSwgMjAxMi8wMi8wNi0xNDo1NjoyNyAgICAgICAgIj4gPHJkZjpSREYgeG1sbnM6cmRmPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5LzAyLzIyLXJkZi1zeW50YXgtbnMjIj4gPHJkZjpEZXNjcmlwdGlvbiByZGY6YWJvdXQ9IiIgeG1sbnM6eG1wPSJodHRwOi8vbnMuYWRvYmUuY29tL3hhcC8xLjAvIiB4bWxuczp4bXBNTT0iaHR0cDovL25zLmFkb2JlLmNvbS94YXAvMS4wL21tLyIgeG1sbnM6c3RSZWY9Imh0dHA6Ly9ucy5hZG9iZS5jb20veGFwLzEuMC9zVHlwZS9SZXNvdXJjZVJlZiMiIHhtcDpDcmVhdG9yVG9vbD0iQWRvYmUgUGhvdG9zaG9wIENTNiAoTWFjaW50b3NoKSIgeG1wTU06SW5zdGFuY2VJRD0ieG1wLmlpZDoxOERBMTRGRDBFMUUxMUUzODUwMkJCOThDMEVFNURFMCIgeG1wTU06RG9jdW1lbnRJRD0ieG1wLmRpZDoxOERBMTRGRTBFMUUxMUUzODUwMkJCOThDMEVFNURFMCI+IDx4bXBNTTpEZXJpdmVkRnJvbSBzdFJlZjppbnN0YW5jZUlEPSJ4bXAuaWlkOjE4REExNEZCMEUxRTExRTM4NTAyQkI5OEMwRUU1REUwIiBzdFJlZjpkb2N1bWVudElEPSJ4bXAuZGlkOjE4REExNEZDMEUxRTExRTM4NTAyQkI5OEMwRUU1REUwIi8+IDwvcmRmOkRlc2NyaXB0aW9uPiA8L3JkZjpSREY+IDwveDp4bXBtZXRhPiA8P3hwYWNrZXQgZW5kPSJyIj8+EwrlwAAAAA5JREFUeNpiMDU0BAgwAAE2AJgB9BnaAAAAAElFTkSuQmCC);
    background-size: 300px 1px;
    color: #lightgrey;
}

.wy-grid-for-nav {
    position: absolute;
    width: 100%;
    height: 100%;
}

.wy-nav-side {
    position: absolute;
    top: 0;
    left: 0;
    width: 300px;
    overflow: hidden;
    min-height: 100%;
    background: #343131;
    z-index: 200;
}

.wy-nav-top {
    display: none;
    background: #2980b9;
    color: #fff;
    padding: 0.4045em 0.809em;
    position: relative;
    line-height: 50px;
    text-align: center;
    font-size: 100%;
    *zoom: 1;
}

.wy-nav-top:before, .wy-nav-top:after {
    display: table;
    content: "";
}

.wy-nav-top:after {
    clear: both;
}

.wy-nav-top a {
    color: #fff;
    font-weight: bold;
}

.wy-nav-top img {
    margin-right: 12px;
    height: 45px;
    width: 45px;
    background-color: #2980b9;
    padding: 5px;
    border-radius: 100%;
}

.wy-nav-top i {
    font-size: 30px;
    float: left;
    cursor: pointer;
}

.wy-nav-content-wrap {
    margin-left: 300px;
    background: #fff;
    min-height: 100%;
}

.wy-nav-content {
    padding: 1.618em 3.236em;
    height: 100%;
    max-width: 1140px;
    margin: auto;
}

.wy-body-mask {
    position: fixed;
    width: 100%;
    height: 100%;
    background: rgba(0, 0, 0, 0.2);
    display: none;
    z-index: 499;
}

.wy-body-mask.on {
    display: block;
}

footer {
    color: #999;
}

footer p {
    margin-bottom: 12px;
}

.rst-footer-buttons {
    *zoom: 1;
}

.rst-footer-buttons:before, .rst-footer-buttons:after {
    display: table;
    content: "";
}

.rst-footer-buttons:after {
    clear: both;
}

#search-results .search li {
    margin-bottom: 24px;
    border-bottom: solid 1px #e1e4e5;
    padding-bottom: 24px;
}

#search-results .search li:first-child {
    border-top: solid 1px #e1e4e5;
    padding-top: 24px;
}

#search-results .search li a {
    font-size: 120%;
    margin-bottom: 12px;
    display: inline-block;
}

#search-results .context {
    color: gray;
    font-size: 90%;
}

@media screen and (max-width: 768px) {
    .wy-body-for-nav {
        background: #fff;
    }

    .wy-nav-top {
        display: block;
    }

    .wy-nav-side {
        left: -300px;
    }

    .wy-nav-side.shift {
        width: 85%;
        left: 0;
    }

    .wy-nav-content-wrap {
        margin-left: 0;
    }

    .wy-nav-content-wrap .wy-nav-content {
        padding: 1.618em;
    }

    .wy-nav-content-wrap.shift {
        position: fixed;
        min-width: 100%;
        left: 85%;
        top: 0;
        height: 100%;
        overflow: hidden;
    }
}

@media screen and (min-width: 1400px) {
    .wy-nav-content-wrap {
        background: #fff;
    }

    .wy-nav-content {
        margin: 0;
        background: #fff;
    }
}

@media print {
    .wy-nav-side {
        display: none;
    }

    .wy-nav-content-wrap {
        margin-left: 0;
    }
}

.rst-versions {
    position: fixed;
    bottom: 0;
    left: 0;
    width: 300px;
    color: #fcfcfc;
    background: #1f1d1d;
    border-top: solid 10px #343131;
    font-family: "Lato", "proxima-nova", "Helvetica Neue", Arial, sans-serif;
    z-index: 400;
}

.rst-versions a {
    color: #2980b9;
    text-decoration: none;
}

.rst-versions .rst-badge-small {
    display: none;
}

.rst-versions .rst-current-version {
    padding: 12px;
    background-color: #272525;
    display: block;
    text-align: right;
    font-size: 90%;
    cursor: pointer;
    color: #27ae60;
    *zoom: 1;
}

.rst-versions .rst-current-version:before, .rst-versions .rst-current-version:after {
    display: table;
    content: "";
}

.rst-versions .rst-current-version:after {
    clear: both;
}

.rst-versions .rst-current-version .icon, .rst-versions .rst-current-version .wy-inline-validate.wy-inline-validate-success .wy-input-context, .wy-inline-validate.wy-inline-validate-success .rst-versions .rst-current-version .wy-input-context, .rst-versions .rst-current-version .wy-inline-validate.wy-inline-validate-danger .wy-input-context, .wy-inline-validate.wy-inline-validate-danger .rst-versions .rst-current-version .wy-input-context, .rst-versions .rst-current-version .wy-inline-validate.wy-inline-validate-warning .wy-input-context, .wy-inline-validate.wy-inline-validate-warning .rst-versions .rst-current-version .wy-input-context, .rst-versions .rst-current-version .wy-inline-validate.wy-inline-validate-info .wy-input-context, .wy-inline-validate.wy-inline-validate-info .rst-versions .rst-current-version .wy-input-context, .rst-versions .rst-current-version .wy-tag-input-group .wy-tag .wy-tag-remove, .wy-tag-input-group .wy-tag .rst-versions .rst-current-version .wy-tag-remove, .rst-versions .rst-current-version .rst-content .admonition-title, .rst-content .rst-versions .rst-current-version .admonition-title, .rst-versions .rst-current-version .rst-content h1 .headerlink, .rst-content h1 .rst-versions .rst-current-version .headerlink, .rst-versions .rst-current-version .rst-content h2 .headerlink, .rst-content h2 .rst-versions .rst-current-version .headerlink, .rst-versions .rst-current-version .rst-content h3 .headerlink, .rst-content h3 .rst-versions .rst-current-version .headerlink, .rst-versions .rst-current-version .rst-content h4 .headerlink, .rst-content h4 .rst-versions .rst-current-version .headerlink, .rst-versions .rst-current-version .rst-content h5 .headerlink, .rst-content h5 .rst-versions .rst-current-version .headerlink, .rst-versions .rst-current-version .rst-content h6 .headerlink, .rst-content h6 .rst-versions .rst-current-version .headerlink, .rst-versions .rst-current-version .rst-content dl dt .headerlink, .rst-content dl dt .rst-versions .rst-current-version .headerlink {
    color: #fcfcfc;
}

.rst-versions .rst-current-version .icon-book {
    float: left;
}

.rst-versions .rst-current-version.rst-out-of-date {
    background-color: #e74c3c;
    color: #fff;
}

.rst-versions.shift-up .rst-other-versions {
    display: block;
}

.rst-versions .rst-other-versions {
    font-size: 90%;
    padding: 12px;
    color: gray;
    display: none;
}

.rst-versions .rst-other-versions hr {
    display: block;
    height: 1px;
    border: 0;
    margin: 20px 0;
    padding: 0;
    border-top: solid 1px #413d3d;
}

.rst-versions .rst-other-versions dd {
    display: inline-block;
    margin: 0;
}

.rst-versions .rst-other-versions dd a {
    display: inline-block;
    padding: 6px;
    color: #fcfcfc;
}

.rst-versions.rst-badge {
    width: auto;
    bottom: 20px;
    right: 20px;
    left: auto;
    border: none;
    max-width: 300px;
}

.rst-versions.rst-badge .icon-book {
    float: none;
}

.rst-versions.rst-badge.shift-up .rst-current-version {
    text-align: right;
}

.rst-versions.rst-badge.shift-up .rst-current-version .icon-book {
    float: left;
}

.rst-versions.rst-badge .rst-current-version {
    width: auto;
    height: 30px;
    line-height: 30px;
    padding: 0 6px;
    display: block;
    text-align: center;
}

@media screen and (max-width: 768px) {
    .rst-versions {
        width: 85%;
        display: none;
    }

    .rst-versions.shift {
        display: block;
    }

    img {
        width: 100%;
        height: auto;
    }
}

.rst-content img {
    max-width: 100%;
    height: auto !important;
}

.rst-content .section>img {
    margin-bottom: 24px;
}

.rst-content a.reference.external:after {
    font-family: fontawesome-webfont;
    content: " \f08e ";
    color: #b3b3b3;
    vertical-align: super;
    font-size: 60%;
}

.rst-content blockquote {
    margin-left: 24px;
    line-height: 24px;
    margin-bottom: 24px;
}

.rst-content .note .last, .rst-content .note p.first, .rst-content .attention .last, .rst-content .attention p.first, .rst-content .caution .last, .rst-content .caution p.first, .rst-content .danger .last, .rst-content .danger p.first, .rst-content .error .last, .rst-content .error p.first, .rst-content .hint .last, .rst-content .hint p.first, .rst-content .important .last, .rst-content .important p.first, .rst-content .tip .last, .rst-content .tip p.first, .rst-content .warning .last, .rst-content .warning p.first {
    margin-bottom: 0;
}

.rst-content .admonition-title {
    font-weight: bold;
}

.rst-content .admonition-title:before {
    margin-right: 4px;
}

.rst-content .admonition table {
    border-color: rgba(0, 0, 0, 0.1);
}

.rst-content .admonition table td, .rst-content .admonition table th {
    background: transparent !important;
    border-color: rgba(0, 0, 0, 0.1) !important;
}

.rst-content .section ol.loweralpha, .rst-content .section ol.loweralpha li {
    list-style: lower-alpha;
}

.rst-content .section ol.upperalpha, .rst-content .section ol.upperalpha li {
    list-style: upper-alpha;
}

.rst-content .section ol p, .rst-content .section ul p {
    margin-bottom: 12px;
}

.rst-content .line-block {
    margin-left: 24px;
}

.rst-content .topic-title {
    font-weight: bold;
    margin-bottom: 12px;
}

.rst-content .toc-backref {
    color: #404040;
}

.rst-content .align-right {
    float: right;
    margin: 0px 0px 24px 24px;
}

.rst-content .align-left {
    float: left;
    margin: 0px 24px 24px 0px;
}

.rst-content h1 .headerlink, .rst-content h2 .headerlink, .rst-content h3 .headerlink, .rst-content h4 .headerlink, .rst-content h5 .headerlink, .rst-content h6 .headerlink, .rst-content dl dt .headerlink {
    display: none;
    visibility: hidden;
    font-size: 14px;
}

.rst-content h1 .headerlink:after, .rst-content h2 .headerlink:after, .rst-content h3 .headerlink:after, .rst-content h4 .headerlink:after, .rst-content h5 .headerlink:after, .rst-content h6 .headerlink:after, .rst-content dl dt .headerlink:after {
    visibility: visible;
    content: "\f0c1";
    font-family: fontawesome-webfont;
    display: inline-block;
}

.rst-content h1:hover .headerlink, .rst-content h2:hover .headerlink, .rst-content h3:hover .headerlink, .rst-content h4:hover .headerlink, .rst-content h5:hover .headerlink, .rst-content h6:hover .headerlink, .rst-content dl dt:hover .headerlink {
    display: inline-block;
}

.rst-content .sidebar {
    float: right;
    width: 40%;
    display: block;
    margin: 0 0 24px 24px;
    padding: 24px;
    background: #f3f6f6;
    border: solid 1px #e1e4e5;
}

.rst-content .sidebar p, .rst-content .sidebar ul, .rst-content .sidebar dl {
    font-size: 90%;
}

.rst-content .sidebar .last {
    margin-bottom: 0;
}

.rst-content .sidebar .sidebar-title {
    display: block;
    font-family: "Roboto Slab", "ff-tisa-web-pro", "Georgia", Arial, sans-serif;
    font-weight: bold;
    background: #e1e4e5;
    padding: 6px 12px;
    margin: -24px;
    margin-bottom: 24px;
    font-size: 100%;
}

.rst-content .highlighted {
    background: #f1c40f;
    display: inline-block;
    font-weight: bold;
    padding: 0 6px;
}

.rst-content .footnote-reference, .rst-content .citation-reference {
    vertical-align: super;
    font-size: 90%;
}

.rst-content table.docutils.citation, .rst-content table.docutils.footnote {
    background: none;
    border: none;
    color: #999;
}

.rst-content table.docutils.citation td, .rst-content table.docutils.citation tr, .rst-content table.docutils.footnote td, .rst-content table.docutils.footnote tr {
    border: none;
    background-color: transparent !important;
    white-space: normal;
}

.rst-content table.docutils.citation td.label, .rst-content table.docutils.footnote td.label {
    padding-left: 0;
    padding-right: 0;
    vertical-align: top;
}

.rst-content table.field-list {
    border: none;
}

.rst-content table.field-list td {
    border: none;
}

.rst-content table.field-list .field-name {
    padding-right: 10px;
    text-align: left;
}

.rst-content table.field-list .field-body {
    text-align: left;
    padding-left: 0;
}

.rst-content tt {
    color: #000;
}

.rst-content tt big, .rst-content tt em {
    font-size: 100% !important;
    line-height: normal;
}

.rst-content tt .xref, a .rst-content tt {
    font-weight: bold;
}

.rst-content dl {
    margin-bottom: 24px;
}

.rst-content dl dt {
    font-weight: bold;
}

.rst-content dl p, .rst-content dl table, .rst-content dl ul, .rst-content dl ol {
    margin-bottom: 12px !important;
}

.rst-content dl dd {
    margin: 0 0 12px 24px;
}

.rst-content dl:not(.docutils) {
    margin-bottom: 24px;
}

.rst-content dl:not(.docutils) dt {
    display: inline-block;
    margin: 6px 0;
    font-size: 90%;
    line-height: normal;
    background: #e7f2fa;
    color: #2980b9;
    border-top: solid 3px #6ab0de;
    padding: 6px;
    position: relative;
}

.rst-content dl:not(.docutils) dt:before {
    color: #6ab0de;
}

.rst-content dl:not(.docutils) dt .headerlink {
    color: #404040;
    font-size: 100% !important;
}

.rst-content dl:not(.docutils) dl dt {
    margin-bottom: 6px;
    border: none;
    border-left: solid 3px #ccc;
    background: #f0f0f0;
    color: gray;
}

.rst-content dl:not(.docutils) dl dt .headerlink {
    color: #404040;
    font-size: 100% !important;
}

.rst-content dl:not(.docutils) dt:first-child {
    margin-top: 0;
}

.rst-content dl:not(.docutils) tt {
    font-weight: bold;
}

.rst-content dl:not(.docutils) tt.descname, .rst-content dl:not(.docutils) tt.descclassname {
    background-color: transparent;
    border: none;
    padding: 0;
    font-size: 100% !important;
}

.rst-content dl:not(.docutils) tt.descname {
    font-weight: bold;
}

.rst-content dl:not(.docutils) .viewcode-link {
    display: inline-block;
    color: #27ae60;
    font-size: 80%;
    padding-left: 24px;
}

.rst-content dl:not(.docutils) .optional {
    display: inline-block;
    padding: 0 4px;
    color: #000;
    font-weight: bold;
}

.rst-content dl:not(.docutils) .property {
    display: inline-block;
    padding-right: 8px;
}

@media screen and (max-width: 480px) {
    .rst-content .sidebar {
        width: 100%;
    }
}

span[id*='MathJax-Span'] {
    color: #404040;
}

.admonition.note span[id*='MathJax-Span'] {
    color: #fff;
}

.admonition.warning span[id*='MathJax-Span'] {
    color: #fff;
}

.search-reset-start {
    color: #463E3F;
    float: right;
    position: relative;
    top: -25px;
    left: -10px;
    z-index: 10;
}

.search-reset-start:hover {
    cursor: pointer;
    color: #2980B9;
}

#search-box-id {
    padding-right: 25px;
}

.DocSiteProduct-header {
    flex: 1;
    -webkit-flex: 1;
    padding: 20px;
    padding-top: 10px;
    padding-bottom: 20px;
    display: flex;
    display: -webkit-flex;
    flex-direction: column;
    -webkit-flex-direction: column;
    align-items: center;
    -webkit-align-items: center;
    justify-content: flex-start;
    -webkit-justify-content: flex-start;
    margin-left: 20px;
    margin-right: 20px;
    text-decoration: none;
    font-weight: 400;
    font-family: 'Open Sans', sans-serif;
}

.DocSiteProduct-header:active,
.DocSiteProduct-header:focus {
    color: #fff;
}

.DocSiteProduct-header:visited {
    color: #fff;
}

.DocSiteProduct-header--core {
    font-size: 25px;
    background-color: #5bbdbf;
    border: 2px solid #5bbdbf;
    border-top-left-radius: 4px;
    border-top-right-radius: 4px;
    color: #fff;
}

.DocSiteProduct-headerAlign {
    width: 100%;
}

.DocSiteProduct-logo {
    width: 60px;
    height: 60px;
    margin-bottom: -9px;
}

.DocSiteProduct-logoText {
    margin-top: 6px;
    font-size: 25px;
}

.DocSite-nav {
    flex: initial;
    -webkit-flex: initial;
    display: flex;
    display: -webkit-flex;
    flex-direction: row;
    -webkit-flex-direction: row;
    justify-content: flex-start;
    -webkit-justify-content: flex-start;
    padding: 15px;
    background-color: #000;
    text-decoration: none;
    font-family: 'Open Sans', sans-serif;
}

.DocSiteNav-logo {
    width: 28px;
    height: 28px;
    margin-right: 8px;
}

.DocSiteNav-title {
    color: #fff;
    font-size: 20px;
}

.DocSiteProduct-CheckVersionPara {
    padding-bottom: 20px;
    margin-left: 10px;
    margin-right: 10px;
}

.wy-menu-vertical a {
    padding: 0;
}

.wy-menu-vertical a.reference.internal {
    padding: 0.4045em 1.618em;
}

.DocSiteBanner {
    display: flex;
    display: -webkit-flex;
    justify-content: center;
    -webkit-justify-content: center;
    flex-wrap: wrap;
    -webkit-flex-wrap: wrap;
    margin-bottom: 25px;
}

.DocSiteBanner-imgWrapper {
    max-width: 100%;
}

th, td {
    min-width: 100px;
}

table {
       overflow-x: auto;
       display: block;
       max-width: 100%;
}

@media print {
    * { background: transparent !important; color: black !important; text-shadow: none !important; filter:none !important; -ms-filter: none !important; }
    #nav,
    a, a:visited { text-decoration: underline; }
    a[href]:after { content: " (" attr(href) ")"; }
    abbr[title]:after { content: " (" attr(title) ")"; }
    .ir a:after, a[href^="javascript:"]:after, a[href^="#"]:after { content: ""; }  /* Don't show links for images, or javascript/internal links */
    pre, blockquote { border: 0px solid #999; page-break-inside: avoid; }
    thead { display: table-header-group; } /* h5bp.com/t */
    tr, img { page-break-inside: avoid; }
    img { max-width: 100% !important; }
    @page { margin: 0.5cm; }
    p, h2, h3 { orphans: 3; widows: 3; }
    h2, h3 { page-break-after: avoid; }
    .DocSiteBanner,
    #google_image_div {
        display: none !important;
    }
}


#sideBanner{
    display: none;
}

.DocSite-globalNav {
   display: none;
}


.DocSite-sideNav {
   display: block;
    margin-bottom: 40px;
}

.ansibleNav {
   background: #000;
   padding: 0px 20px;
   width: auto;
   border-bottom: 1px solid #444;
   font-size: 14px;
}

.ansibleNav ul {
    list-style: none;
    padding-left: 0px;
    margin-top: 0px;
}

    
.ansibleNav ul li{
    padding: 7px 0px;
    border-bottom: 1px solid #444;
}

.ansibleNav ul li:last-child{
    border: none;
}

.ansibleNav ul li a {
    color: #fff;
   text-decoration: none;
   text-transform: uppercase;
   padding: 6px 0px;
}
.ansibleNav ul li a:hover {
    color: #5bbdbf ;
    background: transparent;
}
 
@media screen and (min-width: 768px) {
    
    .DocSite-globalNav{
        display: block
    }
    
    #sideBanner{
        display: block;
    }
    
    .DocSite-sideNav{
        display: none;
    }
    
    .ansibleNav{
        height: 45px;
        width: 100%;
        font-size: 13px;
        padding: 0px 60px 0px 0px;
    }

    .ansibleNav ul {
        float: right;
        display: flex;
        flex-wrap: nowrap;
        margin-top: 13px;
    }
    
        
    .ansibleNav ul li{
        padding: 0px;
        border-bottom: none;
    }
    
    .ansibleNav ul li a {
        color: #fff;
        text-decoration: none;
        text-transform: uppercase;
        padding: 8px 13px;
    }
}

.documentation-table td.elbow-placeholder {
    border-left: 1px solid #000;
    border-top: 0px;
    width: 30px;
    min-width: 30px;
}

.documentation-table th, .documentation-table td {
    padding: 4px;
    border-left: 1px solid #000;
    border-top: 1px solid #000;
}

.documentation-table {
    border-right: 1px solid #000;
    border-bottom: 1px solid #000;
    width: 100%;
    display: table;
}

.caption {
    line-height: 24px;
    margin: 0;
    font-size: 16px;
    margin-bottom: 0px;
    margin-left: 4px;
    color: cornflowerblue;
}

.toctree-l2 {
    margin-left: 16px;
}


.toctree-l3 {
    margin-left: 24px;
}

@import "sphinxrtd.css";
